<?php
/*
 * Plugin Name: Foodle Add-on for Democracy Poll
 * Description: Enhances the Democracy Poll plugin from Kama to interactively display the results of polls in various ways, depending on shortcode parameters, incl. additional functions.
 *
 * Author: Michael Finkenberger
 * Author URI: 
 * Plugin URI: https://de.wordpress.org/plugins/foodle-for-democracy-poll/
 *
 * Text Domain: foodle-for-democracy-poll
 * Domain Path: /languages
 *
 * Requires at least: 4.8
 * Tested up to: 6.1.1
 * Requires PHP: 5.6
 *
 * Licence: GPLv2 or later
 *
 * Version: 1.8.2.0
 * Date: 03.02.2023
*/

if(!defined('ABSPATH')) die(); // no direct access

const FOODLE_VERSION = '1.8.2.0';
global $foodle_title;                // just the administrator's choice
global $foodle_admin_menu_color;     // color value
global $foodle_warnings_removed;     // boolean
global $foodle_help_tooltips;        // boolean
global $foodle_democracy_id;         // to store the Democracy id for the link shortcode inside the poll
global $foodle_undefined_error;      // translated (WordPress action hook 'init')
global $foodle_sorting;              // translated (WordPress action hook 'init')
global $thanks_for_using_foodle;     // translated (WordPress action hook 'init')
global $foodle_like_me_1;            // translated (WordPress action hook 'init')
global $foodle_like_me_2;            // translated (WordPress action hook 'init')
global $foodle_results_text_default; // translated (WordPress action hook 'init')
global $foodle_unknown_user;         // translated (WordPress action hook 'init')
global $foodle_results_text;         // just the administrator's choice or the default



// ************************************************** Options definitions Start **************************************************
//
// Set the user meta fields option 'foodle_meta_fields'.
// •• in front of the usermeta fieldname marks already existing fields to be used and not to be added by Foodle.
// The sub-array contains the fielddescription text, the input ID of the related existing field in the user profile, in case
// it shall be disabled for user input in profile/user-edit/registration, and yes/no to control availability for the polls.
//   array(
//     '••Voice' => array( 'Soprano 1, Soprano 2, Alto 1, Alto 2, Tenor 1, Tenor 2, Bass 1 or Bass 2', 'Voice', 'yes'),
//     'Voicegroup' => array( 'Soprano, Alto, Tenor or Bass', '', 'yes'), // and so on
//   );
//
// Set the user meta fields default values and sorting option 'foodle_meta_defaults_sorting'.
// •• in front of the usermeta fieldname marks already existing fields to be used and not to be added by Foodle.
// The sub-array contains the srttype and the sortlist.
// In parallel, the very meta-name is stored relative to the fieldname in order to allow a back-conversion.
//   array(
//     '••Voice' => array(
//                    'sorttype' => 'list', // 'list' or 'drop-down'
//                    'sortlist' => '<br> Soprano 1<br> Soprano 2<br> Alto 1<br> Alto 2<br> Tenor 1<br> Tenor 2<br> Bass 1<br> Bass 2' // the sorted list of potential entries (if sorttype = 'list', substrings are possible just for sorting)
//                  ), // and so on
//     'Voice' => '••Voice), // and so on
//   );
//
// Set special roles / users (functions) option 'foodle_special_functions'.
//   array( // 'mark' = mark in poll list, 'view' = allow special view, 'no-remind' = disable reminder email
//     'administrator' => array('mark', 'view', 'no-remind'), // Administrator role: mark and special view but no reminders
//     '11'            => array('mark',         'no-remind'), // User ID 11: mark but no reminders, and so on
//   );
//
// Set Foodle reminders option 'foodle_reminders' to store the reminders for each poll.
//   array(
//     Democracy ID => array(
//                       'reminder data 1',
//                       'reminder data 2', // and so on
//                     ), // and so on
//   );
//
// Set regular expression for category inputs derived from the main category content through option 'foodle_regex_main'.
//   array(
//     'Stimmgruppe' => array('/[^a-zA-Z]/g', ''), // Regular Expression modifier: for category 'Stimmgruppe', matches defined by RegExp in array[0] to be replaced by array[1], and so on
//   );
//
// Set the email (raw) content in option 'foodle_email_content'.
//   wp_editor content (sanitized and escaped)
//
// Set other Foodle settings in option 'foodle_settings'.
//   array(
//     'foodle_title' => 'Voodle', // an example name
//     'foodle_results_text' => 'Poll results so far', // an example name
//     'foodle_admin_menu_color' => '#8CBD5A', // and so on
//   );
//
// Set Foodle categories and other options per Democracy Poll ID option 'foodle_dem_categories'.
//   array(
//     Democracy ID               => array(
//                                     'category 1',
//                                     'category 2', // and so on
//                                   ), // and so on
//     'category_column'          => array(
//                                     Democracy ID => 'the category name', // and so on
//                                   ),
//     'sorting_button_text'      => array(
//                                     Democracy ID => 'the very sorting button text', // and so on
//                                   ),
//     'refresh_button_voted'     => array(
//                                     Democracy ID => true/false, // and so on
//                                   ),
//     'refresh_button_not_voted' => array(
//                                     Democracy ID => true/false, // and so on
//                                   ),
//     'count_marked_voters'      => array(
//                                     Democracy ID => true/false, // and so on
//                                   ),
//   );
//
// *************************************************** Options definitions End ***************************************************



// make sure that this plugin is only activated under certain conditions...
function foodle_suppress_activation($plugin, $network_wide) {
  global $wp_version;

  // the same 4 lines as below
  $this_is_me = 'foodle-for-democracy-poll/foodle-for-democracy-poll.php';
  $this_is_my_name = 'Foodle';
  $i_need_this_one = 'democracy-poll/democracy.php';
  $this_is_its_name = 'Democracy Poll';

  $wp_version_min = '4.8';

  $is_not_ok_version = ( $wp_version < $wp_version_min ) ? '<p>WordPress version needs to be at least '.$wp_version_min.' to activate '.$this_is_my_name.'.</p>' : '';
  $is_not_ok_needing = ( ! is_plugin_active( $i_need_this_one ) ) ? '<p>\''.$this_is_its_name.'\' plugin needs to be active to activate '.$this_is_my_name.'.</p>' : '';

  if ( ( $plugin == $this_is_me ) && ( strlen($is_not_ok_version.$is_not_ok_needing) != 0 ) ) {
    error_log( 'The plugin activiation conditions for '.$this_is_my_name.' ('.$this_is_me.') are not met!' );      
    $args = var_export( func_get_args(), true );
    error_log( $args );
    wp_die( $is_not_ok_version.$is_not_ok_needing.'<p><a href="javascript:history.back()">You can press here to return</a></p>' );
  }
}
add_action( 'activate_plugin', 'foodle_suppress_activation' , 10, 2);

// ... and later, just in case: Deactivate this plugin if the plugin activation condtions are no longer met in terms of plugins needed
function foodle_check_activation_conditions() {
  // the same 4 lines as above
  $this_is_me = 'foodle-for-democracy-poll/foodle-for-democracy-poll.php';
  $this_is_my_name = 'Foodle';
  $i_need_this_one = 'democracy-poll/democracy.php';
  $this_is_its_name = 'Democracy Poll';  

  $is_not_ok_needing = '<p>\''.$this_is_its_name.'\' plugin needs to be active to activate '.$this_is_my_name.'.</p>';

  include_once( ABSPATH . 'wp-admin/includes/plugin.php' ); // in order to avoid an 'undefined error message' for is_plugin_active() under circumstances not clear to me...
  if ( ! is_plugin_active( $i_need_this_one ) ) {
    error_log( 'The plugin activiation conditions for '.$this_is_my_name.' ('.$this_is_me.') are not met!' );      
    $args = var_export( func_get_args(), true );
    error_log( $args );
    deactivate_plugins($this_is_me, false, null);
    wp_die( $is_not_ok_needing.'<p><a href="javascript:history.back(); window.location.reload(true);">You can press here to return</a></p>' ); // additional reload(true) to see the change
  }
}
add_action( 'wp_loaded', 'foodle_check_activation_conditions' );



// Activate plugin (conditions check is already done above)
function foodle_for_democracy_poll_activate() {
  delete_option('foodle_variables'); // no longer used from 1.1.8.0 onwards: now stored in user_meta
  // generate the option array for the flag storage when foodle_change_demCollectAnsw() has been replaced in case it doesn't exist, yet
  if ( ! get_option('foodle_change_demCollectAnsw') ) update_option('foodle_change_demCollectAnsw', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_meta_fields') ) update_option('foodle_meta_fields', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_meta_defaults_sorting') ) update_option('foodle_meta_defaults_sorting', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_special_functions') ) update_option('foodle_special_functions', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_reminders') ) update_option('foodle_reminders', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_regex_main') ) update_option('foodle_regex_main', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_email_content') ) update_option('foodle_email_content', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_settings') ) update_option('foodle_settings', array(), 'yes');
  // generate the option array (see above) in case it doesn't exist, yet
  if ( ! get_option('foodle_dem_categories') ) update_option('foodle_dem_categories', array(), 'yes');
}
register_activation_hook( __FILE__, 'foodle_for_democracy_poll_activate' );



// Do some stuff upon plugin deactivation
function foodle_for_democracy_poll_deactivate() {
  // Unschedule Foodle's wp_cron job, in case it is currently scheduled
  $foodle_timestamp = wp_next_scheduled( 'foodle_cron_hook' );
  wp_unschedule_event( $foodle_timestamp, 'foodle_cron_hook' );
}
register_deactivation_hook( __FILE__, 'foodle_for_democracy_poll_deactivate' );



// Clean up upon plugin uninstall
function foodle_for_democracy_poll_uninstall() {
  
  // Delete all options used in the foodle-for-democracy-poll plugin
  delete_option('foodle_variables'); // no longer used from 1.1.8.0 onwards: now stored in user_meta
  delete_option('foodle_change_demCollectAnsw');
  delete_option('foodle_meta_fields');
  delete_option('foodle_meta_defaults_sorting');
  delete_option('foodle_special_functions');
  delete_option('foodle_reminders');
  delete_option('foodle_regex_main');
  delete_option("foodle_email_content");
  delete_option('foodle_settings');
  delete_option('foodle_dem_categories');

  // Delete all foodle meta data
  $users = get_users(array(
    'orderby'  => 'meta_value',
    'meta_key' => 'last_name', // just a habit ;-)
    'order'    => 'ASC'
  ));
  foreach ( $users as $user ) {
    $foodle_user_id = $user->ID;
    $foodle_user_meta = get_user_meta($foodle_user_id);
    foreach( $foodle_user_meta as $foodle_meta_key => $foodle_meta_value ) {
      if ( strpos($foodle_meta_key, 'foodle-') === 0 ) delete_user_meta($foodle_user_id, $foodle_meta_key);
    }
  }

  // Unschedule Foodle's wp_cron job, just in case...
  $foodle_timestamp = wp_next_scheduled( 'foodle_cron_hook' );
  wp_unschedule_event( $foodle_timestamp, 'foodle_cron_hook' );
}
register_uninstall_hook( __FILE__, 'foodle_for_democracy_poll_uninstall' );



// Set title from option
if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['foodle_title']) ) && ( get_option('foodle_settings')['foodle_title'] != '' ) ) {
  $foodle_title = get_option('foodle_settings')['foodle_title'];
} else $foodle_title = 'Foodle';



// Set admin menu color from option...
if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['foodle_admin_menu_color']) ) && ( get_option('foodle_settings')['foodle_admin_menu_color'] != '' ) ) {
  $foodle_admin_menu_color = get_option('foodle_settings')['foodle_admin_menu_color'];
} else $foodle_admin_menu_color = '#8CBD5A';
// ...and in the admin area, mark the admin bar entry for Democracy and the submenu entires for Democracy Poll and Foodle
function modify_admin_submenu_colors() {
  global $foodle_admin_menu_color;
  echo "
    <script type=\"text/javascript\">
      $(document).ready( function() {
        color = '".$foodle_admin_menu_color."';
        $('a[href^=\"options-general.php?page=foodle-admin-page\"]').css('color',color);
        $('a[href^=\"options-general.php?page=democracy-poll\"]').css('color',color);
        $('#wp-admin-bar-dem_settings a').css('color',color);
    });
    </script>
  ";
}
add_action('admin_footer', 'modify_admin_submenu_colors');



function foodle_add_category_selection_to_democracy_poll() {
  global $foodle_title;
  global $foodle_sorting;

  if ( ! get_current_screen()->base == 'settings_page_democracy-poll' ) return; // not the Democracy Poll settings page 

  $foodle_help_tooltips = true; // default: help tooltips are on
  if  ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['help-tooltips']) ) ) {
    $foodle_help_tooltips = get_option('foodle_settings')['help-tooltips']; // check is needed separately here
  }

  $foodle_sorting_button_text = __('Sorting button text','foodle-for-democracy-poll');
  $foodle_show_refresh_button_voted = __('Show refresh button <u>after</u> voting','foodle-for-democracy-poll');
  $foodle_show_refresh_button_not_voted = __('Show refresh button <u>before</u> voting','foodle-for-democracy-poll');
  $foodle_count_marked_voters = __('Count marked voters in the answer column sum','foodle-for-democracy-poll');
  $foodle_delete_poll_IP_addresses = __('Prevent from storing voter IP addresses for this poll','foodle-for-democracy-poll');

  $poll_id_raw = str_replace('&msg=created', '', $_SERVER['REQUEST_URI']);
  $poll_id_start = strpos($poll_id_raw, '&edit_poll=') + 11;
  $poll_id = substr($poll_id_raw, $poll_id_start);

  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['sorting_button_text'][$poll_id]) ) )
    $poll_sorting_button_text = get_option('foodle_dem_categories')['sorting_button_text'][$poll_id];
  else
    $poll_sorting_button_text = '';

  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['refresh_button_voted'][$poll_id]) ) )
    $democracy_refresh_button_voted_checked = ( get_option('foodle_dem_categories')['refresh_button_voted'][$poll_id] ) ? " checked='checked'" : "";
  else
    $democracy_refresh_button_voted_checked = '';
  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['refresh_button_not_voted'][$poll_id]) ) )
    $democracy_refresh_button_not_voted_checked = ( get_option('foodle_dem_categories')['refresh_button_not_voted'][$poll_id] ) ? " checked='checked'" : "";
  else
    $democracy_refresh_button_not_voted_checked = '';
  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['count_marked_voters'][$poll_id]) ) )
    $democracy_count_marked_voters_checked = ( get_option('foodle_dem_categories')['count_marked_voters'][$poll_id] ) ? " checked='checked'" : "";
  else
    $democracy_count_marked_voters_checked = '';
  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['delete_poll_IP_addresses'][$poll_id]) ) )
    $democracy_delete_poll_IP_addresses_checked = ( get_option('foodle_dem_categories')['delete_poll_IP_addresses'][$poll_id] ) ? " checked='checked'" : "";
  else
    $democracy_delete_poll_IP_addresses_checked = '';

  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['category_column'][$poll_id]) ) )
    $poll_category_column = get_option('foodle_dem_categories')['category_column'][$poll_id];
  else
    $poll_category_column = ''; // if empty, use the first category as default category column

  $category_selection = __('Category & Parameter Selection','foodle-for-democracy-poll');
  $foodle_meta_fields = (array)get_option('foodle_meta_fields');

  $foodle_category_column_help = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__('Define the category (see righthand) to be used as category column.<br>Whether displayed or not is being defined in the Foodle shortcode.','foodle-for-democracy-poll')."'" : "";
  $foodle_category_display_help = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__('Define which of the categories to be displayed<br>underneath the answer columns.','foodle-for-democracy-poll')."'" : "";
  $foodle_sortbutton_help = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__('Define the sortbutton text<br>for the Foodle table.','foodle-for-democracy-poll')."'" : "";
  $foodle_refreshbutton_help = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__('Decide whether or not to show the \"refresh\" button in the results display<br>after voting and/or before voting. When e.g. in a simultaneous voting<br>situation, this would refresh the results when pressed.','foodle-for-democracy-poll')."'" : "";
  $foodle_count_marked_voters_help = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__('Decide whether or not to count marked voters<br>in the total sum of an answer column.','foodle-for-democracy-poll')."'" : "";
  $foodle_send_category_selection = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__('Submit your Foodle category & parameter<br>selections with this button.','foodle-for-democracy-poll')."'" : "";
  $foodle_delete_poll_IP_adresses_help = ( $foodle_help_tooltips ) ? " foodle_tooltip='".__("When set, IP addresses will not be stored for this poll, preserving<br>full functionality even when voters have the same IP address.<br>Logging and IP storage of Democracy Poll must remain activated!","foodle-for-democracy-poll")."'" : "";

  $foodle_dem_poll_form = "
  <p id='foodle_dem_poll_form' style='margin:30px; padding:0px;'></p>
    <h2>".$foodle_title." ".$category_selection."</h2>
    <form class='foodle-dem-poll-form' style='margin:0px; padding:10px 20px;' action='".admin_url( 'admin.php?page=foodle-admin-page' )."' method='post'>
    <table style='border-collapse:collapse;'><tbody><tr><th style='text-align:center; padding-right:20px;'".$foodle_category_column_help.">".__('Use as<br>Category Column','foodle-for-democracy-poll')."</th><th".$foodle_category_display_help.">".__('Category display<br>per Answer Column','foodle-for-democracy-poll')."</th></tr>";
  $category_count = -1; // to get 0 after the first iteration
  if ( count($foodle_meta_fields) == 0 ) $foodle_dem_poll_form .= "<tr><td colspan='2'>".__('No meta fields to select from for this poll, yet!','foodle-for-democracy-poll')."</td></tr>";
  foreach($foodle_meta_fields as $foodle_category => $foodle_category_data) {
    $category_count += 1;
    $foodle_category_column_checked = '';
    if ( $poll_category_column == $foodle_category ) $foodle_category_column_checked = "checked='checked'";
    $foodle_category_id = str_replace('.', '€', str_replace(' ', '_', $foodle_category));
    $foodle_category_attr = '';
    $foodle_category_style = '';
    $foodle_label_style = '';
    $foodle_label_txt = '';
    $external_color = 'steelblue';
    $foodle_checked = ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')[$poll_id]) ) && ( in_array($foodle_category, (array)get_option('foodle_dem_categories')[$poll_id]) ) ) ? " checked='checked'" : "";
    if ( $foodle_category_data[2] == 'no' ) {
      $foodle_category_attr = " disabled='disabled'";
      $foodle_category_style = "cursor:default;";
      $foodle_label_style = "color:darkred; cursor:default;";
      $external_color = "darkred";
      $foodle_label_txt =  ( strlen($foodle_checked) > 0 ) ? " (".__('just save once to remove the checkmark','foodle-for-democracy-poll').")" : "";
    }
    if ( strpos($foodle_category, '••') === 0 )
        $foodle_category_display = "<span style='color:".$external_color."'>".str_replace('••', '', $foodle_category)." (".__('existing field','foodle-for-democracy-poll').")</span>";
      else
        $foodle_category_display = $foodle_category;
    $foodle_dem_poll_form .= "
      <tr>
        <td style='text-align:right; padding-right:40px;'".$foodle_category_column_help."><input type='radio' name='foodle_dem_category_column' value='".$foodle_category."' ".$foodle_category_column_checked." /></td>
        <td style='text-align:left; padding-left:20px;'".$foodle_category_display_help."><label style='".$foodle_label_style."'><input type='checkbox' style='".$foodle_category_style."' id='".$foodle_category_id."' name='foodle_dem_categories[]' value='".$foodle_category."' ".$foodle_category_attr.$foodle_checked." />".$foodle_category_display.$foodle_label_txt."</label></td>
      </tr>
    ";
  }
  $foodle_dem_poll_form .= "
    </tbody></table>
    <br>
    <label".$foodle_sortbutton_help."><strong>".$foodle_sorting_button_text."</strong><br><input type='text' name='sorting_button_text' value='".$poll_sorting_button_text."' placeholder='".$foodle_sorting."'/></label>
    <br><br>
    <label".$foodle_refreshbutton_help."><input type='checkbox' name='democracy_refresh_button_voted' value='yes'".$democracy_refresh_button_voted_checked."/><strong>".$foodle_show_refresh_button_voted."</strong></label>
    <br><br>
    <label".$foodle_refreshbutton_help."><input type='checkbox' name='democracy_refresh_button_not_voted' value='yes'".$democracy_refresh_button_not_voted_checked."/><strong>".$foodle_show_refresh_button_not_voted."</strong></label>
    <br><br>
    <label".$foodle_count_marked_voters_help."><input type='checkbox' name='democracy_count_marked_voters' value='yes'".$democracy_count_marked_voters_checked."/><strong>".$foodle_count_marked_voters."</strong></label>
    <br><br>
    <label".$foodle_delete_poll_IP_adresses_help."><input type='checkbox' name='democracy_delete_poll_IP_addresses' value='yes'".$democracy_delete_poll_IP_addresses_checked."/><strong>".$foodle_delete_poll_IP_addresses."</strong></label>
    <br><br><br>
    <input type='hidden' name='democracy_url' value='\" + window.location.href + \"'/>
    <input".$foodle_send_category_selection." type='submit' class='button-primary' name='select_poll_categories' value='".__('Submit category & parameter selection','foodle-for-democracy-poll')."...'/>
    </form>
  ";
  $foodle_dem_poll_announce = "
  <p style='margin:40px; padding:0px;'></p>
    <h2>".$foodle_title." ".$category_selection."</h2>
    <p style='margin:0px; padding:10px 20px;'>".__('Category and parameter selection will be available, once the poll was created','foodle-for-democracy-poll')."</p>
  ";
  echo '
    <script type="text/javascript">
        var $ = jQuery;
        foodle_href_check = window.location.href;
        $(document).ready( function() {
            $(".new-poll-answers").before("<p style=\'margin:0px 25px; padding:0px;\'>('.__('Use','foodle-for-democracy-poll').' •• '.__('as first two digits of an answer to identify it as \'radio\' selection inside a \'multiple answers\' poll','foodle-for-democracy-poll').')</p>");
            $("input[name=dmc_multiple]").parent().append("&nbsp;('.__('The use of •• as first two digits of an answer will make a difference if this option is active!','foodle-for-democracy-poll').')");
            $foodle_log_message="'.foodle_check_keeping_logs().'";
            if ( foodle_href_check.indexOf("&subpage=add_new") == -1 ) {
                $(".dem-new-poll").before($foodle_log_message);
                $(".dempolls").before($foodle_log_message);
                $(".demlogs").before($foodle_log_message);
                $(".dempage_settings").before($foodle_log_message);
                $(".dem-new-poll").before("<p>&nbsp;</p><p><a class=\"button-primary foodle-scroll\" style=\"background-color:SlateGrey;\" href=\"#foodle_dem_poll_form\">☟ '.__("Down to the Foodle settings","foodle-for-democracy-poll").'</a></p>");
                $(".dem-new-poll").after("'.preg_replace('/\s+/', ' ', $foodle_dem_poll_form).'");
            }
            else {
                $(".dem-new-poll").before($foodle_log_message);
                $(".dem-new-poll").after("'.preg_replace('/\s+/', ' ', $foodle_dem_poll_announce).'");
            }
            $(".foodle-scroll").on("click", function(event) {
                if (this.hash !== "") {
                    event.preventDefault();
                    var hash = this.hash;
                    $("html, body").animate({
                        scrollTop: $(hash).offset().top
                    }, 1000, function(){
                        window.location.hash = hash;
                    });
                } // End if
            });
        });
    </script>
  ';
}
add_action('admin_footer', 'foodle_add_category_selection_to_democracy_poll', 999);



// Initialize the individual foodle variables prior their usage upon page load in order to
// avoid the array getting bigger and bigger with the foodle private tokens above 20KB
function foodle_initialize_foodle_variables_in_user_meta() {
  $foodle_current_user_id = get_current_user_id();
  $foodle_variable_size = strlen(serialize(get_user_meta($foodle_current_user_id, 'foodle_variables', true)));
  if ( $foodle_variable_size > 20480 )
    update_user_meta($foodle_current_user_id, 'foodle_variables', array());
}
add_action('set_current_user', 'foodle_initialize_foodle_variables_in_user_meta');



// Introduce AJAX call to foodle display function after democracy poll screen change and introduce a few adaptations
function foodle_democracy_update( $html, $foodle_this ) {
  global $wpdb;
  
  $foodle_user_id = get_current_user_id();
  $foodle_id = $foodle_this->id;
  $foodle_variables = (array)get_user_meta($foodle_user_id, 'foodle_variables', true);
  echo '<script type="text/javascript">
    var $ = jQuery;'; // open script
  if ( isset($foodle_variables[$foodle_id]) ) foreach( $foodle_variables[$foodle_id] as $foodle_private_token => $dummy ) {
    $gl_ajshow_date = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajshow_date"];
    $gl_ajshow_category = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajshow_category"];
    $gl_ajdem_categories = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajdem_categories"];
    $gl_maxlines_single_summary = $foodle_variables[$foodle_id][$foodle_private_token]["gl_maxlines_single_summary"];
    $gl_ajanswerlist = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajanswerlist"];
    $gl_ajcategorysort = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajcategorysort"];
    $gl_ajblocksort = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajblocksort"];
    $gl_ajsolo = $foodle_variables[$foodle_id][$foodle_private_token]["gl_ajsolo"];
    $gl_maxcount = $foodle_variables[$foodle_id][$foodle_private_token]["gl_maxcount"];
    if ( ! get_the_ID() === false ) {
      $foodle_the_post_id = get_the_ID();
      $foodle_variables[$foodle_id][$foodle_private_token]["foodle_the_post_id"] = $foodle_the_post_id;
      update_user_meta($foodle_user_id, 'foodle_variables', $foodle_variables);
    } else
      if ( isset(get_user_meta($foodle_user_id, 'foodle_variables', true)[$foodle_id][$foodle_private_token]["foodle_the_post_id"]) ) {
        $foodle_the_post_id = get_user_meta($foodle_user_id, 'foodle_variables', true)[$foodle_id][$foodle_private_token]["foodle_the_post_id"];
    } else {
      $foodle_the_post_id = 0;
    }
  
    $update = "";
    $not_post = array();
    if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['foodle_page_post_exclusion']) ) && ( get_option('foodle_settings')['foodle_page_post_exclusion'] != '' ) ) $not_post = (array)get_option('foodle_settings')['foodle_page_post_exclusion'];
    if ( !in_array( $foodle_the_post_id, $not_post ) ) {
      $update = '
        if ( $("#mf_sw_foodle_sort_content_'.$foodle_private_token.'").length > 0 ) mf_sw_foodle_sort_js('.$foodle_private_token.','.$gl_ajshow_date.','.$gl_ajshow_category.','.$gl_ajdem_categories.','.$gl_maxlines_single_summary.','.$gl_ajanswerlist.','.$gl_ajcategorysort.','.$gl_ajblocksort.','.$gl_ajsolo.','.$gl_maxcount.');';
    }
    echo $update;
  }
  echo '
    foodle_radio(); </script>'; // close script
  $html = str_replace('<form ', '<form id="mf_foodle_form" ', $html); // Mark the Democracy Poll form with a unique ID
  $foodle_fnscript = '
    <script type="text/javascript">
      var $ = jQuery;
      $("#mf_foodle_form").parent().parent().mouseenter( function () {
        ';
  if ( ! get_option('foodle_change_demCollectAnsw')) {
    $foodle_fnscript .= 'foodle_change_demCollectAnsw();'; // Replace the Democracy Poll function, adding mixed polls (checkbox and radio)
    update_option('foodle_change_demCollectAnsw', 'true', 'no');
  }
  $foodle_refresh = __('Refresh','foodle-for-democracy-poll');
  $foodle_refresh_pic = plugin_dir_url(__FILE__).'img/refresh.png';
  $foodle_fnscript .= '
      });
        $.each( $(".dem-label, .dem__radio_label"), function( i, foodle_result_label){
        var foodle_this = $(foodle_result_label);
        mf_label_html = foodle_this.html();
        mf_search = mf_label_html.search("••");
        if ( mf_search != -1 ) {
          mf_label_html = mf_label_html.replace("••", "");
          foodle_this.html(mf_label_html);
        }
      });
      $("#democracy-'.$foodle_id.' .dem-voted-this .dem-label").append("<div class=\"democracy-your-vote\">('.__("You voted here","foodle-for-democracy-poll").')</div");
      if ( $("#foodle-democracy-top-spacing-'.$foodle_id.'").length == 0 ) $("#democracy-'.$foodle_id.'").before("<p class=\'foodle-democracy-top-spacing\' id=\'foodle-democracy-top-spacing-'.$foodle_id.'\'></p>");

      $(".democracy .dem-vote").closest(".democracy").removeClass("democracy-results").addClass("democracy-voting");
      $(".democracy .dem-answers").closest(".democracy").removeClass("democracy-voting").addClass("democracy-results");
      $(".democracy").removeClass("democracy-voted").addClass("democracy-not-voted");
      $(".democracy .dem-voted-this").closest(".democracy").removeClass("democracy-not-voted").addClass("democracy-voted");';
  if ( ( $foodle_this->poll->open == 1 ) && ( isset(get_option('foodle_dem_categories')['refresh_button_voted'][$foodle_id])) && ( get_option('foodle_dem_categories')['refresh_button_voted'][$foodle_id]) ) $foodle_fnscript .= '
      $("#democracy-'.$foodle_id.'.democracy-results.democracy-voted .dem-poll-info").append("<button type=\"submit\" margin-top:5px; style=\"float:right; box-shadow:5px 5px 5px SteelBlue; font-size:0.65em;\" class=\"dem-button democracy-refresh-button\" data-dem-act=\"view\">'.$foodle_refresh.'&nbsp;&nbsp;<img src=\"'.$foodle_refresh_pic.'\" width=\"18\"></button>");';
  if ( ( $foodle_this->poll->open == 1 ) && ( isset(get_option('foodle_dem_categories')['refresh_button_not_voted'][$foodle_id])) && ( get_option('foodle_dem_categories')['refresh_button_not_voted'][$foodle_id]) ) $foodle_fnscript .= '
      $("#democracy-'.$foodle_id.'.democracy-results.democracy-not-voted .dem-poll-info").append("<button type=\"submit\" margin-top:5px; style=\"float:right; box-shadow:5px 5px 5px SteelBlue; font-size:0.65em;\" class=\"dem-button democracy-refresh-button\" data-dem-act=\"view\">'.$foodle_refresh.'&nbsp;&nbsp;<img src=\"'.$foodle_refresh_pic.'\" width=\"18\"></button>");';
  $foodle_fnscript .= '
    </script>'; // add foodle-democracy-top-spacing only once in case it doesn't exist, yet, and add some classes for use with CSS

  // Delete the poll voters' IP addresses - if requested for this poll
  if ( ( get_option('foodle_dem_categories') ) && ( isset(get_option('foodle_dem_categories')['delete_poll_IP_addresses'][$foodle_id]) ) )
  if ( get_option('foodle_dem_categories')['delete_poll_IP_addresses'][$foodle_id] ) {
    $wpdb->query("UPDATE {$wpdb->democracy_log} SET ip = NULL WHERE qid = {$foodle_id}");
  }

  return $html.$foodle_fnscript;  // return $html.$update.$foodle_fnscript;  
}
add_filter( 'dem_vote_screen', 'foodle_democracy_update', 10, 2 );
add_filter( 'dem_result_screen', 'foodle_democracy_update', 10, 2 );



// enable a shortcode inside the democracy shortcode (purpose: to allow e.g. 'foodle-link-democracy-poll' short-
// code to be used inside the Democracy text field underneath the voting/results)
function foodle_execute_shortcode_inside_democracy($output, $tag, $attr) {
  global $foodle_democracy_id;

  $foodle_democracy_id = 0; // to initialize
  if('democracy' == $tag) {
    if (  isset($attr['id']) ) $foodle_democracy_id = $attr['id']; // store for use with [foodle-link-democracy-poll id="self"]
    $output = do_shortcode($output);
    $foodle_democracy_id = 0; // we're out now
    return $output;
  } else {
    return $output;
  }
}
add_filter( 'do_shortcode_tag','foodle_execute_shortcode_inside_democracy', 10, 3 );



// Schedule, unschedule and execute wp_cron jobs for Foodle
// Set a new bi-monthly interval
function foodle_bi_monthly_interval( $schedules ) { 
    $schedules['two_months'] = array(
        'interval' => 60 * 60 * 24 * 30 * 2, // 2 months in seconds
        'display'  => esc_html__( 'Every second month', 'foodle-for-democracy-poll' ),
    );
    return $schedules;
}
add_filter( 'cron_schedules', 'foodle_bi_monthly_interval' );
// schedule the wp_cron task if not already done (to be called when needed)
function foodle_schedule_cron() {
  if ( ! wp_next_scheduled( 'foodle_cron_hook' ) ) {
      wp_schedule_event( time(), 'two_months', 'foodle_cron_hook' );
  }
}
// unschedule the wp_cron task (to be called when needed)
function foodle_unschedule_cron() {
  $foodle_timestamp = wp_next_scheduled( 'foodle_cron_hook' );
  wp_unschedule_event( $foodle_timestamp, 'foodle_cron_hook' );
  // set all democracy poll's answer expiry dates to one year from the answer date
  foodle_set_democracy_poll_answer_expiry_dates('vote_date');
}
// this is the very task done every two months
function foodle_execute_cron() {
  // set all democracy poll's answer expiry dates to one year from now
  foodle_set_democracy_poll_answer_expiry_dates('now');
}
add_action( 'foodle_cron_hook', 'foodle_execute_cron' );
// Now do the job as requested
function foodle_set_democracy_poll_answer_expiry_dates($timeframe) {
  global $wpdb;
  $database_prefix = $wpdb->prefix;
  
  if ( ( $timeframe == 'vote_date' ) || ( $timeframe == 'now' ) ) {
    if ( $timeframe == 'now' ) {
      $one_year_timestamp = date_i18n('U') + 60 * 60 * 24 * 365; // one year in seconds from today
      $sql = "UPDATE $wpdb->democracy_log SET expire={$one_year_timestamp}";
      $wpdb->query($sql); 
    } else {
      foreach($wpdb->get_results("SELECT * FROM $wpdb->democracy_log") as $poll_vote ) { // get the relevant poll data
        $poll_voted = $poll_vote->date;
        $poll_logid = $poll_vote->logid;
        $poll_vote_expire = strtotime($poll_voted) + 60 * 60 * 24 * 365;
        $sql = "UPDATE $wpdb->democracy_log SET expire={$poll_vote_expire}  where logid={$poll_logid}";
        $wpdb->query($sql);
      }
    }
  }
}



delete_option('foodle_change_demCollectAnsw'); // Mark an internal Democracy Poll function as not yet amended (used for mixed polls)



// Load the plugin's textdomain to retrieve the translations
function foodle_translations() {

  $domain = 'foodle-for-democracy-poll';
/* * /
  $mfem_locale = apply_filters( 'plugin_locale', determine_locale(), $domain );
  $mfem_mofile = $domain . '-' . $mfem_locale . '.mo'; // determine the correct .mo-file
  $mfem_mopath = WP_PLUGIN_DIR . '/' . basename( dirname( __FILE__ ) ) . '/languages/';
  unload_textdomain( $domain ); // make sure that the WordPress translations are unloaded
  if ( !load_textdomain( $domain, $mfem_mopath . $mfem_mofile ) ) // Load the very plugin translations (if they exist for $mfem_locale) as the WP SVN version synchronization seems failing...
    load_plugin_textdomain( $domain, false, basename( dirname( __FILE__ ) ) . '/languages/' ); // else: load WordPress plugin translations from /wp-content/languages/plugins/
/**/
  load_plugin_textdomain( $domain, false, basename( dirname( __FILE__ ) ) . '/languages/' ); // load WordPress plugin translations with normal logic
 }
add_action( 'init', 'foodle_translations' );



function foodle_translate_global_strings () {
  global $foodle_undefined_error;      // translated
  global $foodle_sorting;              // translated
  global $thanks_for_using_foodle;     // translated
  global $foodle_like_me_1;            // translated
  global $foodle_like_me_2;            // translated
  global $foodle_results_text_default; // translated
  global $foodle_unknown_user;         // translated
  global $foodle_results_text;         // just the administrator's choice or the default

  $foodle_sorting = __('Sorting','foodle-for-democracy-poll'); // default text e.g. for the foodle category column sorting
  $foodle_undefined_error = __('Undefined error','foodle-for-democracy-poll').'!'; // Generic error message used more than once
  $thanks_for_using_foodle = __('Thanks a lot for using Foodle','foodle-for-democracy-poll');
  $foodle_like_me_1 = __('If you like this plugin, you could ','foodle-for-democracy-poll');
  $foodle_like_me_2 = __(' your appreciation...','foodle-for-democracy-poll');
  $foodle_results_text_default = __('Poll results so far','foodle-for-democracy-poll');
  $foodle_unknown_user = __('Unknown User','foodle-for-democracy-poll');


  // Set table headline from option
  if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['foodle_results_text']) ) && ( get_option('foodle_settings')['foodle_results_text'] != '' ) ) {
    $foodle_results_text = get_option('foodle_settings')['foodle_results_text'];
  } else $foodle_results_text = $foodle_results_text_default;
}
add_action('init', 'foodle_translate_global_strings');



// Change Admin Footer Text for Foodle
function foodle_update_admin_footer_text($content) {
  global $thanks_for_using_foodle;
  if ( strpos(get_current_screen()->base, 'foodle-admin-page') !== false ) {
    remove_all_filters( 'admin_footer_text' );
    return $thanks_for_using_foodle.'!';
  }
  else
    return $content;
}
add_filter( 'admin_footer_text', 'foodle_update_admin_footer_text', 1, 1); // Priority 1 to make it work



// Change Admin Footer Update for Foodle
function foodle_update_admin_footer_version($content) {
  if ( strpos(get_current_screen()->base, 'foodle-admin-page') !== false ) {
    remove_all_filters( 'update_footer' );
    return 'Foodle Version '.FOODLE_VERSION;
  }
  else
    return $content;
}
add_filter( 'update_footer', 'foodle_update_admin_footer_version', 1, 1); // Priority 1 to make it work



// Enqueue back-end "Touch Punch" for touchscreen use and at the same time ensure jquery-ui-sortable is active
function hook_foodle_touch_punch_script( $foodle_hook ) {
  if ( $foodle_hook == 'settings_page_foodle-admin-page' ) { // only in 'Foodle'
    $js_url = plugin_dir_url(__FILE__).'js/foodle_jquery.ui.touch-punch.js';
    wp_register_script( 'foodle-touch-punch-script', $js_url, array('jquery','jquery-ui-core','jquery-ui-sortable','jquery-ui-widget','jquery-ui-mouse'), FOODLE_VERSION, false );
    wp_enqueue_script( 'foodle-touch-punch-script' );
  }
}
add_action( 'admin_enqueue_scripts', 'hook_foodle_touch_punch_script' ); // for the admin area



// Enqueue back-end plugin style-file
function add_back_end_foodle_stylesheet( $foodle_hook ) {
  if ( $foodle_hook == 'settings_page_foodle-admin-page' ) { // only in 'Foodle'
    $css_url = plugin_dir_url(__FILE__).'styles/foodle-for-democracy-poll-back-end.css';
    wp_register_style( 'foodle-back-end-css', $css_url, array(), FOODLE_VERSION, 'all' );
    wp_enqueue_style( 'foodle-back-end-css' );
  }
}
add_action( 'admin_enqueue_scripts', 'add_back_end_foodle_stylesheet' ); // for the admin area



// enqueue back-end and front-end tooltips js
function hook_foodle_tooltips_script() {
  $js_url = plugin_dir_url(__FILE__).'js/foodle_tooltips.js';
  wp_register_script( 'foodle-tooltips-script', $js_url, array('jquery'), FOODLE_VERSION, true ); // true = 'in_footer' to reach Democracy Poll from here
  wp_enqueue_script( 'foodle-tooltips-script' );
}
add_action( 'admin_enqueue_scripts', 'hook_foodle_tooltips_script' ); // for the admin area
add_action( 'wp_enqueue_scripts', 'hook_foodle_tooltips_script' ); // for the front-end as well


// enqueue back-end and front-end tooltips style-file
function hook_foodle_tooltips_stylesheet() {
  $css_url = plugin_dir_url(__FILE__).'styles/foodle-tooltips.css';
  wp_register_style( 'foodle-tooltips-css', $css_url, array(), FOODLE_VERSION, 'all' );
  wp_enqueue_style( 'foodle-tooltips-css' );
}
add_action( 'admin_enqueue_scripts', 'hook_foodle_tooltips_stylesheet' ); // for the admin area
add_action( 'wp_enqueue_scripts', 'hook_foodle_tooltips_stylesheet' ); // for the front-end as well



// Enqueue front-end plugin style-file
function add_front_end_foodle_stylesheet() {
  $css_url = plugin_dir_url(__FILE__).'styles/foodle-for-democracy-poll-front-end.css';
  wp_register_style( 'foodle-front-end-css', $css_url, array(), FOODLE_VERSION, 'all' );
  wp_enqueue_style( 'foodle-front-end-css' );
}
add_action( 'wp_enqueue_scripts', 'add_front_end_foodle_stylesheet' ); // for the front-end



// Enqueue front-end plugin script to convert checkboxes marked with &bull; into a radio button
function foodle_hook_radioconvert_script() {
  $js_url = plugin_dir_url(__FILE__).'js/foodle_radio.js';
  wp_register_script( 'foodle_radioconvert', $js_url, array('jquery'), FOODLE_VERSION, false);
  wp_enqueue_script( 'foodle_radioconvert' );
}
add_action( 'wp_enqueue_scripts', 'foodle_hook_radioconvert_script' ); // for the front-end



// Description: Front-end AJAX-Function, displaying the Democracy Poll results from its database interactively.
function foodle_update_hook_ajax_script() {
  $js_url = plugin_dir_url(__FILE__).'js/foodle_update_sort_ajax_file.js';
  wp_register_script( 'foodle_update_ajax_script', $js_url, array('jquery'), FOODLE_VERSION, false );
  wp_enqueue_script( 'foodle_update_ajax_script' );

  wp_localize_script( 'foodle_update_ajax_script', 'foodle_update_ajax_var', array(
    'ajaxurl' => admin_url( 'admin-ajax.php' ),
    'title' => get_the_title()
    )
  );
}
add_action( 'wp_enqueue_scripts', 'foodle_update_hook_ajax_script' ); // for the front-end



// Add the submenu entry and the help tabs...
function add_foodle_admin_menu_items()
{
  $foodle_admin_page = add_submenu_page(
                          'options-general.php', // related menu
                          'Foodle for Democracy Poll', // page title
                          'Foodle ➤ Democracy', // submenu title
                          'manage_options', // required capability (or role)
                          'foodle-admin-page', // page slug
                          'foodle_admin_page_callback', // callback function, the main plugin function
                          10 // position not defined, yet
                        );
  
  add_action( 'load-'.$foodle_admin_page, 'foodle_add_help' ); // Add the help tabs callback function...
}
add_action('admin_menu','add_foodle_admin_menu_items');



// Add the help tabs and sidebar here...
function foodle_add_help () {
  global $version;
  global $foodle_like_me_1;
  global $foodle_like_me_2;

  $mfem_screen = get_current_screen();
  
  // Generic explanations
  $id = 'foodle_gen_ex';
  $title = __('Information','foodle-for-democracy-poll');
  $content  = '<p>'.__('The following \'help\' tabs may be consulted for detailed explanations and hints for their related topic\'s usage.</p>
               <p>More detailed information is available in the <a href="/wp-admin/options-general.php?page=foodle-admin-page&tab=foodle-tips">Tips tab</a> or by activating \'Help-Tooltips\' in the <a href="/wp-admin/options-general.php?page=foodle-admin-page&tab=foodle-settings">Foodle Settings tab</a>.</p>
               <p>Additional information may be added in the future to display important Q&As.','foodle-for-democracy-poll').'</p>';
  $content .= '<p>&nbsp;</p>
               <p style="font-size:1.2em;">'.$foodle_like_me_1.'<img src="'.plugin_dir_url( __FILE__ ).'img/PayPal-Logo-2019-kl.jpg" height="16" alt="PayPal">'.$foodle_like_me_2.'</p>
               <form action="https://www.paypal.com/donate" method="post" target="_top">
               <input type="hidden" name="hosted_button_id" value="W3V5CKXFJS948" />
               <input type="image" src="https://www.paypalobjects.com/en_US/DK/i/btn/btn_donateCC_LG.gif" border="0" name="submit" title="PayPal - The safer, easier way to pay online!" alt="Donate with PayPal button" />
               <img alt="" border="0" src="https://www.paypal.com/en_DE/i/scr/pixel.gif" width="1" height="1" />
               </form>';

  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );

  // Foodle Shortcode and email option
  $id = 'foodle_short_code';
  $title = __('Foodle Shortcode and Email Option','foodle-for-democracy-poll');
  $content = '<p>'.__('The Foodle shortcode is used to display the poll results in various ways. It can be used on the same page as the democracy poll shortcode and will be have interactively (AJAX) when voting. However, it can as well be used independently on any page or post.</p>
              <p>You may consult the <a href="/wp-admin/options-general.php?page=foodle-admin-page&tab=foodle-tips">Tips tab</a> for shortcode details.</p>
              <p>For <a href="/wp-admin/options-general.php?page=foodle-admin-page&tab=special-roles-users">selectable viewers</a>, the shortcode provides an automated email reminder option for those who didn\'t vote, yet. The email text can be edited in the <a href="https://finkenberger-net.voicesunlimited.de/wp-admin/options-general.php?page=foodle-admin-page&tab=edit-email">related tab</a>. Placeholders are available to configure the email in many ways. This includes name information of the person that will trigger the email (can be anyone of those special viewers having access).','foodle-for-democracy-poll').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );

  // Questions & Answers
  $id = 'foodle_q_a';
  $title = __('Questions & Answers','foodle-for-democracy-poll');
  $content = '<p>'.__('Nothing asked, yet...','foodle-for-democracy-poll').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // The help sidebar
  $mfem_sidebar =  '<p style="font-weight:bold;">'.__('Support Hints','foodle-for-democracy-poll').'</p>';
  $mfem_sidebar .= '<p>'.__('For plugin information, please consult the','foodle-for-democracy-poll');
  $mfem_sidebar .= ' <a href="https://de.wordpress.org/plugins/foodle-for-democracy-poll/" target="_blank">';
  $mfem_sidebar .= __('WordPress&nbsp;Plugin&nbsp;Page','foodle-for-democracy-poll').'</a>.</p>';
  $mfem_sidebar .= '<p>'.__('In case of questions or issues, you may use the','foodle-for-democracy-poll');
  $mfem_sidebar .= ' <a href="https://wordpress.org/support/plugin/foodle-for-democracy-poll/" target="_blank">';
  $mfem_sidebar .= __('Plugin&nbsp;Supportpage','foodle-for-democracy-poll').'</a> ';
  $mfem_sidebar .= __('or contact the plugin author directly under','foodle-for-democracy-poll').' <a href="mailto:plugins@finkenberger.net?subject=Foodle for Democracy Poll Plugin Direct Support">';
  $mfem_sidebar .= __('Direct&nbsp;Support','foodle-for-democracy-poll').'</a>.</p>';
  $mfem_sidebar .= '<p>V'.FOODLE_VERSION.'</p>';
  $mfem_screen->set_help_sidebar( $mfem_sidebar );
}



// Adding WordPress plugin action links 
function foodle_add_plugin_action_links( $links ) {
  $foodle_color = '';
  if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['foodle_admin_menu_color']) ) )
    $foodle_color = get_option('foodle_settings')['foodle_admin_menu_color'];
  else
    $foodle_color = '#8CBD5A'; // the default
  return array_merge(
    array(
      'settings' => '<a style="font-size:1.1em; color:'.$foodle_color.'"; href="' . get_bloginfo( 'wpurl' ) . '/wp-admin/options-general.php?page=foodle-admin-page">'.__('Foodle-Settings','foodle-for-democracy-poll').'</a>'
    ),
    $links
  );
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'foodle_add_plugin_action_links' );



require('foodle-extra-fields.php'); // add the requested additional fields to each user's profile

require('foodle-utilities.php'); // the main shortcode definition and supporting functions in separate file

require('foodle-shortcode-tools.php'); // other shortcodes in separate file

require('foodle-send-reminder.php'); // email function enqueuing and sending in separate file

require('foodle-display-on-for-roles.php'); // shortcode (with separate end tag) to limit enclosed content for specific roles

require('foodle-archive-do-not-show.php'); // shortcode (with separate end tag) to not display certain ids in an archive listing



function foodle_ajax_update_php() {
  global $foodle_undefined_error;

  $out = $foodle_undefined_error;

  if ( isset($_POST["id"]) && isset($_POST["answerlist"]) && isset($_POST["categorysort"]) ) {
    
    $id           = sanitize_text_field($_POST["id"]);         // The foodle's id
    $showdate     = sanitize_text_field($_POST["showdate"]);   // The boolean value string whether to show the participation date (true) or not (false)
    $showcategory = sanitize_text_field($_POST["showcategory"]);  // The boolean value string whether to show the participant's category (true) or not (false)
    $categories   = sanitize_text_field($_POST["categories"]);     // The boolean value string whether to add a count of categorys per foodle answer (true)
    $mlss         = sanitize_text_field($_POST["mlss"]);       // The max number of returns with only one row providing a sum per answer
    $answerlist   = sanitize_text_field($_POST["answerlist"]); // The boolean value string whether to display doodle-like (false) or as an answerlist (true)
    $categorysort = sanitize_text_field($_POST["categorysort"]);  // The boolean value string whether to sort by categorys (true) or date of entry (false)
    $blocksort    = sanitize_text_field($_POST["blocksort"]);  // The boolean value string whether to change the sorting is blocked (true) or not (false)
    $solo         = sanitize_text_field($_POST["solo"]);       // The boolean value string whether to display less, i.e. just the detailed answers (true)
    $maxcount     = sanitize_text_field($_POST["maxcount"]);   // The maximum number of responses selected in a foodle (0 or 0,y = ignore)
    
    $out = do_shortcode("[foodle-democracy-poll-list-log id='".$id."' show_date='".$showdate."' show_category='".$showcategory."' categories='".$categories."' ml_single_sum='".$mlss."' answerlist='".$answerlist."' categorysort='".$categorysort."' blocksort='".$blocksort."' solo='".$solo."' maxcount='".$maxcount."' ajax='true']");
    
  } else { $out = "<div style='color:red;'>".__('Error!<br />Essential parameters for shortcode execution are missing!','foodle-for-democracy-poll')."</div>"; }

  echo $out;
  die();
}
add_action( 'wp_ajax_nopriv_foodle_ajax_update_php', 'foodle_ajax_update_php' );
add_action( 'wp_ajax_foodle_ajax_update_php', 'foodle_ajax_update_php' );



// ************************************************************************************************************************************************************************************
// ************************************************************************************************************************************************************************************
// ************************************************************************************************************************************************************************************
// **************************************************************************                              ****************************************************************************
// **************************************************************************   The main plugin function   ****************************************************************************
// **************************************************************************                              ****************************************************************************
// ************************************************************************************************************************************************************************************
// ************************************************************************************************************************************************************************************
// ************************************************************************************************************************************************************************************

function foodle_admin_page_callback() {
  global $wpdb;
  global $foodle_title;
  global $foodle_admin_menu_color;
  global $foodle_undefined_error;
  global $foodle_sorting;
  global $foodle_warnings_removed;
  global $foodle_help_tooltips;
  global $foodle_results_text_default;
  global $foodle_results_text;

  if ( ! get_option('foodle_dem_categories') ) update_option('foodle_dem_categories', array(), 'yes'); // just to be sure

  $update_in_progress = __('Update in progress ','foodle-for-democracy-poll'); // default text when jumping to another admin page

  $foodle_warnings_removed = false; // default: warnings are not off
  if  ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['remove-warnings']) ) ) {
    $foodle_warnings_removed = get_option('foodle_settings')['remove-warnings'];
  }

  $foodle_help_tooltips = true; // default: help tooltips are on
  if  ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['help-tooltips']) ) ) {
    $foodle_help_tooltips = get_option('foodle_settings')['help-tooltips'];
  }

  // check user capabilities
  if ( ! current_user_can( 'manage_options' ) ) {
    return;
  }

  // handle and store the Foodle category selection from the democracy poll edit page in option 'foodle_dem_categories'
  if ( isset($_POST['select_poll_categories']) ) {
    $foodle_admin_notice = '';
    $foodle_error = false;

    if ( isset($_POST['democracy_url']) ) {
      $democracy_url = filter_var(esc_url_raw($_POST['democracy_url']), FILTER_VALIDATE_URL);
      $new_poll = __('New Poll','foodle-for-democracy-poll');
      if (! $democracy_url === false ) {
        if ( substr( $democracy_url, strpos( $democracy_url, '&subpage=') + 9 ) == 'add_new' )
          $poll_id = $new_poll;
        else
          $poll_id = str_replace('&msg=created', '', substr( $democracy_url, strpos( $democracy_url, '&edit_poll=') + 11 ));
        if ( $poll_id != $new_poll ) {
          $foodle_dem_categories = (array)get_option('foodle_dem_categories');
          if ( isset($_POST['foodle_dem_categories']) ) {
            $foodle_dem_categories[$poll_id] = array(); // initialize
            foreach($_POST['foodle_dem_categories'] as $foodle_dem_category) {
              $foodle_dem_categories[$poll_id][] = sanitize_text_field($foodle_dem_category);
            }
          } else {
              $foodle_dem_categories[$poll_id] = array(); // no categories
          }
          if ( isset($_POST['sorting_button_text']) ) // determine and store the sorting button text per Democracy Poll id
            $foodle_dem_categories['sorting_button_text'][$poll_id] = sanitize_text_field($_POST['sorting_button_text']);
          else
            $foodle_dem_categories['sorting_button_text'][$poll_id] = $foodle_sorting;
          if ( isset($_POST['democracy_refresh_button_voted']) ) // determine whether the refresh button shall be displayed after voting per Democracy Poll id
            $foodle_dem_categories['refresh_button_voted'][$poll_id] = ( $_POST['democracy_refresh_button_voted'] == 'yes' );
          else
            $foodle_dem_categories['refresh_button_voted'][$poll_id] = false;
          if ( isset($_POST['democracy_refresh_button_not_voted']) ) // determine whether the refresh button shall be displayed before voting per Democracy Poll id
            $foodle_dem_categories['refresh_button_not_voted'][$poll_id] = ( $_POST['democracy_refresh_button_not_voted'] == 'yes' );
          else
            $foodle_dem_categories['refresh_button_not_voted'][$poll_id] = false;
          if ( isset($_POST['democracy_count_marked_voters']) ) // determine whether the refresh button shall be displayed before voting per Democracy Poll id
            $foodle_dem_categories['count_marked_voters'][$poll_id] = ( $_POST['democracy_count_marked_voters'] == 'yes' );
          else
            $foodle_dem_categories['count_marked_voters'][$poll_id] = false;
          if ( isset($_POST['democracy_delete_poll_IP_addresses']) ) // determine whether IP addresses shall be deleted upon voting per Democracy Poll id
            $foodle_dem_categories['delete_poll_IP_addresses'][$poll_id] = sanitize_text_field($_POST['democracy_delete_poll_IP_addresses']);
            else
            $foodle_dem_categories['delete_poll_IP_addresses'][$poll_id] = false;
          if ( isset($_POST['foodle_dem_category_column']) ) // determine and store the category column per Democracy Poll id
            $foodle_dem_categories['category_column'][$poll_id] = sanitize_text_field($_POST['foodle_dem_category_column']);
          else
            $foodle_dem_categories['category_column'][$poll_id] = '';
        }
        foodle_pwx_admin_notice__warning('<p><span style="vertical-align:160%;">'.$update_in_progress.'...&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span> <img src="'.plugin_dir_url(__FILE__).'img/loader.gif"></p>');
        update_option('foodle_dem_categories', $foodle_dem_categories, 'yes'); //store it ...
        echo '<script type="text/javascript">window.location.href="'.$democracy_url.'"</script>'; // ... and jump back to democracy poll edit

        if ( $foodle_error ) { // currently not used...
          foodle_pwx_admin_notice__defined_error('<p>'.$foodle_admin_notice.'</p>');
        } else { // we should never come here...
           foodle_pwx_admin_notice__success('<p>'.__('The Foodle categories settings were properly stored for poll id ','foodle-for-democracy-poll').$poll_id.'.</p>');
        }
      } else {
        foodle_pwx_admin_notice__defined_error('<p>'.__('The democracy poll url could not properly be retrieved','foodle-for-democracy-poll').'</p>');
      }
    } else {
      foodle_pwx_admin_notice__defined_error('<p>'.__('The Foodle categories could not properly be set during democracy poll edit','foodle-for-democracy-poll').'</p>');
    }
  }

  // handle and store the other settings in option 'foodle_settings'
  if ( isset($_POST['save_foodle_settings']) ) {
    $foodle_admin_notice = '';
    $foodle_error = false;

    if ( ( isset($_POST['foodle_title']) ) && ( $_POST['foodle_title'] != '' ) ) {
      $foodle_title = sanitize_text_field($_POST['foodle_title']);
    } else $foodle_title = 'Foodle';

    if ( ( isset($_POST['foodle_results_text']) ) && ( $_POST['foodle_results_text'] != '' ) ) {
      $foodle_results_text = sanitize_text_field($_POST['foodle_results_text']);
    } else $foodle_results_text = $foodle_results_text_default;

    if ( ( isset($_POST['foodle_admin_menu_color']) ) && ( $_POST['foodle_admin_menu_color'] != '' ) ) {
      $foodle_admin_menu_color = sanitize_text_field($_POST['foodle_admin_menu_color']);
    } else $foodle_admin_menu_color = '#8CBD5A';

    if ( ( isset($_POST['foodle_page_post_exclusion']) ) && ( $_POST['foodle_page_post_exclusion'] != '' ) ) {
      $foodle_page_post_exclusion = array_filter(explode(',', sanitize_text_field($_POST['foodle_page_post_exclusion'])));
    } else $foodle_page_post_exclusion = array();

    if ( isset($_POST['remove_vote_expiry']) ) {
      $foodle_remove_vote_expiry = true;
      foodle_schedule_cron();
    } else {
      $foodle_remove_vote_expiry = false;
      foodle_unschedule_cron();
    }

    if ( isset($_POST['remove-warnings']) ) {
      $foodle_warnings_removed = true;
    } else {
      $foodle_warnings_removed = false;
    }

    if ( isset($_POST['help-tooltips']) ) {
      $foodle_help_tooltips = true;
    } else {
      $foodle_help_tooltips = false;
    }

    $foodle_settings = array();
    $foodle_settings['foodle_title'] = $foodle_title;
    $foodle_settings['foodle_results_text'] = $foodle_results_text;
    $foodle_settings['foodle_admin_menu_color'] = $foodle_admin_menu_color;
    $foodle_settings['remove_vote_expiry'] = $foodle_remove_vote_expiry;
    $foodle_settings['remove-warnings'] = $foodle_warnings_removed;
    $foodle_settings['help-tooltips'] = $foodle_help_tooltips;
    $foodle_settings['foodle_page_post_exclusion'] = $foodle_page_post_exclusion;

    update_option('foodle_settings', $foodle_settings, 'yes');

    if ( $foodle_error ) {
      foodle_pwx_admin_notice__defined_error('<p>'.$foodle_admin_notice.'</p>');
    } else {
       foodle_pwx_admin_notice__success('<p>'.__('The Foodle settings were properly stored','foodle-for-democracy-poll').'.</p>');
    }
  }

  // handle and store the user / role properties in option 'foodle_special_functions'
  if ( isset($_POST['save_special_roles_users']) ) {
    $foodle_admin_notice = 'Error: No error found - but nothing done!';
    $foodle_error = false;
    $foodle_special_functions = array();
    if ( ( isset($_POST['foodle_functions_line']) ) && ( isset($_POST['roles_users_field']) ) ) {
      $foodle_line_count = -1; // to have the first iteration see 0
      foreach( $_POST['foodle_functions_line'] as $foodle_line_ref ) { // just for referring to the input lines, even if some had been deleted
        $foodle_line_count += 1;
        $foodle_line_ref = sanitize_text_field($foodle_line_ref);

        if ( ( $_POST['roles_users_field'][$foodle_line_count] == 'no_selection' ) ) continue; // remove lines without user or role selection but allow lines without a capability being set

        $roles_users_field = sanitize_text_field($_POST['roles_users_field'][$foodle_line_count]);

        $foodle_capabilities_raw = ( isset($_POST['capabilities'.$foodle_line_ref]) ) ? $_POST['capabilities'.$foodle_line_ref] : '' ;
        $foodle_capabilities = array();
        foreach( (array)$foodle_capabilities_raw as $foodle_capability ) {
          $foodle_capabilities[] = sanitize_text_field($foodle_capability);
        }

        $foodle_special_functions[$roles_users_field] = $foodle_capabilities;
      }
      update_option('foodle_special_functions', $foodle_special_functions, 'yes');

      if ( $foodle_error ) {
        foodle_pwx_admin_notice__defined_error('<p>'.$foodle_admin_notice.'</p>');
      } else {
        foodle_pwx_admin_notice__success('<p>'.__('The Foodle special roles & users definiton was properly set','foodle-for-democracy-poll').'.</p>');
      }
    } else {
      foodle_pwx_admin_notice__error();
    }
  }

  // handle the meta field defaults & sorting definitions and storage in option 'foodle_defaults_sorting' and replacements directly in database 'usermeta'
  if ( ( isset($_GET['foodle_sorting_done']) ) && ( $_GET['foodle_sorting_done'] == 'yes' ) ) { //coming properly back from changing the data
    $foodle_admin_notice = '<p>'.__('The Foodle Defaults & Sorting definition was properly done','foodle-for-democracy-poll').'.</p>';
    foodle_pwx_admin_notice__success($foodle_admin_notice);
  }
  if ( isset($_POST['save_meta_defaults_sorting']) ) {
    $foodle_admin_notice = '';
    $foodle_error = false;
    $foodle_meta_defaults_sorting = array();
    if ( isset($_POST['subreplace']) ) {
      foreach($_POST['subreplace'] as $foodle_meta_name => $foodle_meta_name_replace_data) {
        $foodle_meta_name = sanitize_text_field($foodle_meta_name);
        foreach($foodle_meta_name_replace_data as $foodle_subcategory_old => $foodle_subcategory_new) {
          $foodle_subcategory_old = sanitize_text_field($foodle_subcategory_old);
          $foodle_subcategory_new = sanitize_text_field($foodle_subcategory_new);
          if ( $foodle_subcategory_new != '') {
            $foodle_meta_key=foodle_fieldname_to_meta_name($foodle_meta_name);
            $sql = "UPDATE $wpdb->usermeta SET meta_value='".$foodle_subcategory_new."' WHERE meta_key='".$foodle_meta_key."' AND meta_value='".$foodle_subcategory_old."'";
            $wpdb->query($sql);
          }
        }
      }
      $wpdb->flush(); // flush any cached query result
    }
    if ( isset($_POST['sorttype']) ) {
      foreach($_POST['sorttype'] as $foodle_meta_name => $foodle_meta_sort_type) {
        $foodle_meta_name = sanitize_text_field($foodle_meta_name);
        $foodle_meta_sort_type = sanitize_text_field($foodle_meta_sort_type);
        if ( isset($_POST['sortlist'][$foodle_meta_name]) ) {
          $foodle_meta_sort_list = '';
          if ( isset($_POST['sortlist']) ) {
              $foodle_meta_sort_list = nl2br($_POST['sortlist'][$foodle_meta_name],false);
              $foodle_meta_sort_list = html_entity_decode(sanitize_text_field(htmlentities(stripslashes(str_replace('<br><br>','<br>',preg_replace('#\r|\n#','',$foodle_meta_sort_list))))));
            }
          $foodle_meta_defaults_sorting[$foodle_meta_name]['sorttype'] = $foodle_meta_sort_type;
          $foodle_meta_defaults_sorting[$foodle_meta_name]['sortlist'] = $foodle_meta_sort_list;
          $foodle_meta_defaults_sorting[foodle_fieldname_to_meta_name($foodle_meta_name)] = $foodle_meta_name;
        }
      }
    }
    update_option('foodle_meta_defaults_sorting', $foodle_meta_defaults_sorting, 'yes');
    foodle_pwx_admin_notice__warning('<p><span style="vertical-align:160%;">'.$update_in_progress.'...&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span> <img src="'.plugin_dir_url(__FILE__).'img/loader.gif"></p>');
    echo '<script type="text/javascript">window.location.href="'.admin_url( 'admin.php?page=foodle-admin-page&tab=metafield-default-sorting&foodle_sorting_done=yes' ).'"</script>'; // ... and jump back to properly reload the user meta data
  }

  // handle the complex user meta and regexp handling and storage in options 'foodle_meta_fields' and 'foodle_regex_main'
  if ( isset($_POST['save_metafields_definition']) ) {
    $foodle_admin_notice = '';
    $foodle_error = false;
    $foodle_meta_fields = array();
    $foodle_regex_main = array();
    if ( ( isset($_POST['foodle_meta_line']) ) && ( isset($_POST['own_meta_field']) ) && ( isset($_POST['existing_meta_field']) ) && ( isset($_POST['description']) ) && ( isset($_POST['regexp_field']) ) && ( isset($_POST['foodle_poll_use0']) ) ) {
      $foodle_line_count = -1; // to have the first iteration see 0
      foreach( $_POST['foodle_meta_line'] as $foodle_line_ref ) { // just for referring to the input lines, even if some had been deleted
        $foodle_line_count += 1;
        $foodle_line_ref = sanitize_text_field($foodle_line_ref);
        if ( $_POST['own_meta_field'][$foodle_line_count] == '---' ) {
          $foodle_field_name = '••'.sanitize_text_field($_POST['existing_meta_field'][$foodle_line_count]);
        } else {
            $foodle_field_name = sanitize_text_field($_POST['own_meta_field'][$foodle_line_count]);
        }
        $description = sanitize_text_field($_POST['description'][$foodle_line_count]);
        $description = ($description == '---') ? '' : $description;
        $link_meta_field = sanitize_text_field($_POST['link_meta_field'][$foodle_line_count]);
        $link_meta_field = ($link_meta_field == '---') ? '' : $link_meta_field;
        $foodle_poll_use = sanitize_text_field($_POST['foodle_poll_use'.$foodle_line_ref]);

        if ( $foodle_field_name != '' ) $foodle_meta_fields[$foodle_field_name] = array($description, $link_meta_field, $foodle_poll_use);
        $regexp_field = ( $foodle_line_count == 0 ) ? '---' : sanitize_text_field($_POST['regexp_field'][$foodle_line_count]); // if - after moving - the first row would carry such a content
        $regrep_field = ( $foodle_line_count == 0 ) ? '---' : sanitize_text_field($_POST['regrep_field'][$foodle_line_count]); // if - after moving - the first row would carry such a content
        $regrep_field = ($regrep_field == '---') ? '' : $regrep_field;
        if ( ( $regexp_field != '---' ) && ( $foodle_field_name != '' ) ) { 
          $foodle_regex_main[$foodle_field_name] = array($regexp_field, $regrep_field);
        }
      }
      update_option('foodle_meta_fields', $foodle_meta_fields, 'yes');
      update_option('foodle_regex_main', $foodle_regex_main, 'yes');

      if ( $foodle_error ) {
        foodle_pwx_admin_notice__defined_error('<p>'.$foodle_admin_notice.'</p>');
      } else {
        foodle_pwx_admin_notice__success('<p>'.__('The Foodle meta fields definiton was properly set','foodle-for-democracy-poll').'.</p>');
      }
    } else {
      foodle_pwx_admin_notice__error();
    }
  }

  // In case the eMail text was set or updated: check if eMail text is sufficient (min. one word with min. three letters) or needs to be updated and store it as option 'mfem_email_content'
  if ( isset($_POST['foodle_email_content']) ) {
    $foodle_admin_notice = '';
    $foodle_new_email_content = sanitize_text_field(htmlentities(stripslashes($_POST['foodle_email_content'])));
    if ( ( $foodle_new_email_content <> '' ) && ( $foodle_new_email_content === get_option('foodle_email_content') ) ) {
      $foodle_admin_notice .= __('The email content didn\'t change.','foodle-for-democracy-poll');
      foodle_pwx_admin_notice__success('<p>'.$foodle_admin_notice.'</p>');
    }
    else {
      if ( ( preg_match('/[a-zA-Z]{3}/', $foodle_new_email_content) )
          && ( update_option('foodle_email_content', $foodle_new_email_content) ) ) {
        $foodle_admin_notice .= __('The email content was properly stored.','foodle-for-democracy-poll');
        foodle_pwx_admin_notice__success('<p>'.$foodle_admin_notice.'</p>');
      }
      else {
        $foodle_admin_notice .= __('The email content wasn\'t stored due to text missing (looking for at least one word with three letters minimum...)!','foodle-for-democracy-poll');
        foodle_pwx_admin_notice__defined_error('<p>'.$foodle_admin_notice.'</p>');
      }
    }
  }

  // Get the active tab from the $_GET parameter
  $default_tab = 'define-metafields';
  $tab = ( isset($_GET['tab']) ) ? sanitize_text_field($_GET['tab']) : $default_tab;

  // Admin page should be inside a 'wrap': opening div
  echo '<div class="wrap">';

  // Display the page title
  echo '<h1 class="mfem-h1">'.esc_html( get_admin_page_title() ).'</h1>';
  echo '<h2 class="mfem-h2">'.__('An add-on to display poll responses in various ways and much more...','foodle-for-democracy-poll').'</h2>';
  
  // Check whether Democracy will keep logs. Otherwise, issue a warning!
  echo foodle_check_keeping_logs();
  
  ?><!-- Here are the tabs -->
    <nav class="nav-tab-wrapper">
      <a href="?page=foodle-admin-page&tab=define-metafields" class="nav-tab <?php if($tab==='define-metafields'): ?>nav-tab-active<?php endif; ?>"><?php _e('Define Metafields','foodle-for-democracy-poll')?></a>
      <a href="?page=foodle-admin-page&tab=metafield-default-sorting" class="nav-tab <?php if($tab==='metafield-default-sorting'): ?>nav-tab-active<?php endif; ?>"><?php _e('Meta Field Defaults & Sorting','foodle-for-democracy-poll')?></a>
      <a href="?page=foodle-admin-page&tab=special-roles-users" class="nav-tab <?php if($tab==='special-roles-users'): ?>nav-tab-active<?php endif; ?>"><?php _e('Special Roles & Users','foodle-for-democracy-poll')?></a>
      <a href="?page=foodle-admin-page&tab=edit-email" class="nav-tab <?php if($tab==='edit-email'): ?>nav-tab-active<?php endif; ?>"><?php _e('Edit Email','foodle-for-democracy-poll')?></a>
      <a href="?page=foodle-admin-page&tab=foodle-settings" class="nav-tab <?php if($tab==='foodle-settings'): ?>nav-tab-active<?php endif; ?>"><?php _e('Foodle Settings','foodle-for-democracy-poll')?></a>
      <a href="?page=foodle-admin-page&tab=foodle-tips" class="nav-tab <?php if($tab==='foodle-tips'): ?>nav-tab-active<?php endif; ?>"><?php _e('Tips','foodle-for-democracy-poll')?></a>
    </nav>

    <div class="tab-content">
    <?php switch($tab) :
      case 'define-metafields':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Define the Meta Fields for the User Profile','foodle-for-democracy-poll').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','foodle-for-democracy-poll').')</p>';
        echo '<p>&nbsp;</p>';
        foodle_define_metafields();
        break;
      case 'metafield-default-sorting':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Define Meta Field Default Values and their Sorting','foodle-for-democracy-poll').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','foodle-for-democracy-poll').')</p>';
        echo '<p>&nbsp;</p>';
        foodle_metafield_defaults_sorting();
        break;
        case 'special-roles-users':
          echo '<p>&nbsp;</p>';
          echo '<h2>'.__('Select Special Roles and / or Users for Poll Marking, Hidden Views and eMail Non-Reminders','foodle-for-democracy-poll').'</h2>';
          echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','foodle-for-democracy-poll').')</p>';
          echo '<p>&nbsp;</p>';
          foodle_set_special_roles_users();
          break;
        case 'edit-email':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Edit the Reminder Email for "Lazybones"','foodle-for-democracy-poll').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','foodle-for-democracy-poll').')</p>';
        echo '<p>&nbsp;</p>';
        foodle_edit_email();
        break;
      case 'foodle-settings':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Other Settings to Control Foodle','foodle-for-democracy-poll').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','foodle-for-democracy-poll').')</p>';
        echo '<p>&nbsp;</p>';
        foodle_settings();
        break;
        case 'foodle-tips':
          echo '<p>&nbsp;</p>';
          echo '<h2>'.__('Tips for using Foodle','foodle-for-democracy-poll').'</h2>';
          echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','foodle-for-democracy-poll').')</p>';
          echo '<p>&nbsp;</p>';
          foodle_tips();
          break;
        default:
        echo 'Not covered, yet!';
        break;
    endswitch; ?>
    </div>

  <?php

  echo '<p>&nbsp;</p>';

  // Admin page should be inside a 'wrap': closing div
  echo '</div>';

}



function foodle_define_metafields() {
  global $wpdb;
  global $wp_roles;
  global $foodle_warnings_removed;
  global $foodle_help_tooltips;

  // Warnings and Help Tooltips
  $warning_meta_existing_metafield_name = ( ! $foodle_warnings_removed ) ? ' foodle_ttwarning="'.__('Warning:<br>Be careful when working with unknown existing fields.<br>This may harm WordPress\' or other plugins\' functionality!','foodle-for-democracy-poll').'" ' : '';
  $help_meta_own_metafield_name = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('You may define your own fieldnames to be filled in the user\'s profiles.<br>They can be used to display categorized voter information in polls<br>or even be used independently from this plugin.<br>(Content is being validated during input)<br><br><strong>Whether own or existing meta field:</strong><br>The meta field name will be displayed as field title in the user profile.<br>Sorting is always done in the same order as the fields are defined in this table!','foodle-for-democracy-poll').'" ' : '';
  $help_meta_regexp_def = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('With the help of Regular Expressions, fields can be filled automatically in the user\'s profile,<br>based on future entries in the first user meta field (even when the first field is filled using<br>drop-down). Therefore, this option is only available for the second and further fields.<br>However, Regular Expressions for fields will be overruled, when the same field is defined<br>as a drop-down field for user entry!<br>(Content is being validated partially during input and the RegExp prior submit)','foodle-for-democracy-poll').'" ' : '';
  $help_meta_existing_metafield_name = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Alternatively, you may chose already existing fields (maybe,<br>another plugin brought them for you before).<br>This plugin tries to eliminate unwanted existing user-meta content<br>from being displayed here.<br><span style=\'color:darkred;\'>However, you must take extreme special care when using such<br>existing user meta fields as the results may not be foreseeable!</span><br><br><strong>Whether own or existing meta field:</strong><br>The meta field name will be displayed as field title in the user profile.<br>Sorting is always done in the same order as the fields are defined in this table!','foodle-for-democracy-poll').'" ' : '';
  $help_meta_regexp_repl = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Once a Regular Expression match is found (if defined in the lefthand input),<br>this is the replacement string. Leave empty to delete the matched string.<br>\'---\' will be entered automatically throughout this table for empty inputs.<br>(Content is being validated during input)','foodle-for-democracy-poll').'" ' : '';
  $help_meta_link_input = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('If you determine the id of a related existing input field in the user profile,<br>you can herewith link this existing input with your own input field.<br>The other input field will be deactivated and a remark will be displayed.<br>(Content is being validated during input)','foodle-for-democracy-poll').'" ' : '';
  $help_meta_poll_allow = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Determines, whether the field can be activated as category per answer column in the<br>democracy poll edit page. If not, the field will be shown but be disabled for selection.<br>It is used there to select the categories shown underneath the poll\'s answer columns.<br>The definition as category column in the Foodle table is not concerned here.','foodle-for-democracy-poll').'" ' : '';
  $help_meta_description = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('You can define a description to be displayed alongside the input field in the user profile.<br>This could be used to explain the field use or the expected input format (if not defined anyway through drop-down).<br>(Content is being validated during input)','foodle-for-democracy-poll').'" ' : '';
  $help_meta_delete_row = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Delete this meta field row.','foodle-for-democracy-poll').'"' : '';
  $help_meta_add_row = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Add a new meta field row.','foodle-for-democracy-poll').'"' : '';
  $help_dragging_rows = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('You may drag each row to where you want in order to change the row order.<br>When moving the first row or when moving another row there,<br>a new row will take that special place <strong>after saving</strong>.','foodle-for-democracy-poll').'<br><span style=\'color:DarkGoldenRod;\'>'.__('In such case, any RegExp content will be lost in that new first row.','foodle-for-democracy-poll').'</span>"' : '';

  $DB_PREFIX = $wpdb->prefix;
  $users = get_users(array(
    'orderby'  => 'meta_value',
    'meta_key' => 'last_name', // just a habit ;-)
    'order'    => 'ASC'
  ));
  $all_user_meta = array();
  foreach( $users as $user ) {
    $user_metas = array_keys(get_user_meta($user->ID));
    foreach( $user_metas as $user_meta ) {
      if (
        ( ! in_array ( $user_meta, array(
          'first_name','last_name','nickname','description','rich_editing','comment_shortcuts',
          'admin_color','jabber','aim','yim','default_password_nag','use_ssl','show_admin_bar_front',
          'show_welcome_panel','dismissed_wp_pointers','nav_menu_recently_edited','managenav-menuscolumnshidden') )
        )
        && ( ! is_array(get_user_meta($user->ID,$user_meta,true) ) ) // avoid any user meta with array content
        && ( strpos($user_meta,'closedpostboxes_') === false )
        && ( strpos($user_meta,'metaboxhidden_') === false )
        && ( strpos($user_meta,'meta-box-order_') === false )
        && ( strpos($user_meta,'screen_layout_') === false )
        && ( strpos($user_meta,$DB_PREFIX) === false )
        && ( strpos($user_meta,'foodle') === false )
        && ( strpos($user_meta,'meta') === false )
        && ( strpos($user_meta,'session') === false )
        && ( strpos($user_meta,'locale') === false )
        && ( strpos($user_meta,'syntax') === false )
        && ( strpos($user_meta,'mfem') === false )
        && ( strpos($user_meta,'mfdwc') === false )
      ) $all_user_meta[$user_meta] = true; // each user meta key exists here only once
    }
  }
  $all_user_meta = array_keys($all_user_meta);
  sort($all_user_meta); // a sorted list of all existing user meta keys except the foodle ones and except the database and WP related ones

  echo '<p><form action="'.admin_url( 'admin.php?page=foodle-admin-page&tab=define-metafields' ).'" method="post">';
  ?>
    <figure class="foodle-block-table">
      <table unselectable="on" class="foodle-meta-table unselectable">
        <thead>
          <tr class="foodle-header-row">
            <th style="padding:5px; text-align:center;"><div style="opacity:0.4; margin:auto; width:18px; height:16px; background-size:contain; background-image: url('<?php echo plugin_dir_url(__FILE__).'img/move_sm.png'; ?>');" <?php echo $help_dragging_rows ?>></div></th>
            <th style="padding:5px; text-align:center;"><br><?php echo __('Use Own Meta Fields','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:5px; text-align:center;"><br><?php echo __('Use Existing Meta Fields<br>(WP-filtered)','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:5px; text-align:center;"><br><?php echo __('Avoid Existing Input','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:5px; text-align:center;"><br><?php echo __('Add Input Description','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:5px; text-align:center;"></th> <!-- the column with the buttons -->
          </tr>
          <tr class="foodle-meta-row-above">
            <td colspan="6" style="text-align:center;">
              <?php echo __('This first row is the basis for any other fields\' user-defined content fed by regular expressions','foodle-for-democracy-poll').'!'; ?>
            </td>
          </tr>
        </thead>
        <tbody id="foodle-meta-sortable" style="overflow-y:auto;">
  <?php
  if ( get_option('foodle_meta_fields') ) {
    $foodle_meta_fields = get_option('foodle_meta_fields');
  } else {
    $foodle_meta_fields[''] = array('', '', 'yes');
  }
  if ( get_option('foodle_regex_main') ) {
    $foodle_regex_main = get_option('foodle_meta_fields');
  } else {
    $foodle_regex_main['---'] = array('','');
  }

  $foodle_number_of_rows = count($foodle_meta_fields);
  $foodle_meta_count = -1; // in order to have the first iteration see 0
  foreach ( $foodle_meta_fields as $foodle_meta_field_raw => $foodle_meta_data ) {
    $foodle_meta_count += 1;
    if ( strpos($foodle_meta_field_raw, '••') === 0 ) {
      $foodle_meta_field = '---';
      $foodle_own_field_style='opacity:0.4; cursor:not-allowed;';
      $foodle_own_field_attr='readonly="readonly"';
      $foodle_existing_field = str_replace('••', '', $foodle_meta_field_raw);
      $foodle_link_meta_field_style='opacity:1.0;';
      $foodle_link_meta_field_attr='';
    } else {
      $foodle_meta_field = $foodle_meta_field_raw;
      $foodle_own_field_style='opacity:1.0;';
      $foodle_own_field_attr='';
      $foodle_existing_field = 'own-field';
      $foodle_link_meta_field_style='opacity:0.4; cursor:not-allowed;';
      $foodle_link_meta_field_attr='readonly="readonly"';
    }
    $foodle_link_meta_field = ( $foodle_meta_data[1] == '' ) ? '---' : $foodle_meta_data[1];
    $foodle_description = ( $foodle_meta_data[0] == '' ) ? '---' : $foodle_meta_data[0];
    $poll_use_yes_checked = ( $foodle_meta_data[2] == 'yes' ) ? 'checked="checked"' : '';
    $poll_use_no_checked = ( $foodle_meta_data[2] == 'no' ) ? 'checked="checked"' : '';
    $column_category_checked = '';
    $required = ''; // no requirement at this time, maybe in a later plugin version
    if ($foodle_meta_count == 0) {
      $foodle_background = 'background-color:LightBlue;';
      $foodle_del_button_style = 'opacity:0.1; cursor:not-allowed;';
      $foodle_del_button_attr = 'disabled="disabled"';
      if ( $foodle_number_of_rows == 1 ) { // just one row existing
        $foodle_add_button_style = 'opacity:1.0; cursor:pointer;';
        $foodle_add_button_attr = '';
      } else {
        $foodle_add_button_style = 'opacity:0.1; cursor:not-allowed;';
        $foodle_add_button_attr = 'disabled="disabled"';
      }
      $foodle_regexp_field = '---';
      $foodle_regexp_style = 'opacity:0.2; cursor:not-allowed;';
      $foodle_regexp_attr = 'readonly="readonly"';
      $foodle_regrep_field = '---';
      $foodle_regrep_style = 'opacity:0.2; cursor:not-allowed;';
      $foodle_regrep_attr = 'readonly="readonly"';
    } else {
      $foodle_background = 'background-color:#dddddd;';
      $foodle_del_button_style = 'opacity:1.0; cursor:pointer;';
      $foodle_del_button_attr = '';
      if ( $foodle_meta_count + 1 == $foodle_number_of_rows ) { // the last row
        $foodle_add_button_style = 'opacity:1.0; cursor:pointer;';
        $foodle_add_button_attr = '';
      } else {
        $foodle_add_button_style = 'opacity:0.1; cursor:not-allowed;';
        $foodle_add_button_attr = 'disabled="disabled"';
      }
      $foodle_regexp_style = 'opacity:1.0;';
      $foodle_regexp_attr = '';
      $foodle_regrep_style = 'opacity:1.0;';
      $foodle_regrep_attr = '';
      if ( get_option('foodle_regex_main') ) {
        $foodle_regexp_meta_field = str_replace('', '', $foodle_meta_field_raw); // optional later str_replace
        if ( isset(get_option('foodle_regex_main')[$foodle_regexp_meta_field]) ) {
          $foodle_regexp_field = get_option('foodle_regex_main')[$foodle_regexp_meta_field][0];
          $foodle_regexp_field = ( $foodle_regexp_field == '' ) ? '---' : $foodle_regexp_field;
          $foodle_regrep_field = get_option('foodle_regex_main')[$foodle_regexp_meta_field][1];
          $foodle_regrep_field = ( $foodle_regrep_field == '' ) ? '---' : $foodle_regrep_field;
        } else {
          $foodle_regexp_field = '---';
          $foodle_regrep_field = '---';
        }
      } else {
      $foodle_regexp_field = '---';
      $foodle_regrep_field = '---';
      }
    }
    $not_used = __('not used','foodle-for-democracy-poll');

    if ($foodle_meta_count == 0 ): ?>
          <tr class="foodle-meta-row" style="border-top: 0px solid #888888; <?php echo $foodle_background; ?>" id="foodle-meta-row-<?php echo $foodle_meta_count ?>">
    <?php
    else:
    ?>
          <tr class="foodle-meta-row" style="border-top: 1px solid #888888; <?php echo $foodle_background; ?>" id="foodle-meta-row-<?php echo $foodle_meta_count ?>">
    <?php
    endif
    ?>
            <td style="padding: 10px 10px;">
              <div class="foodle-meta-handle" style="cursor:move; margin:auto; width:18px; height:16px; background-size:contain; background-image: url('<?php echo plugin_dir_url(__FILE__).'img/move_sm.png'; ?>');" <?php echo $help_dragging_rows ?>></div>
            </td>
            <td style="padding: 10px 10px;">
              <label <?php echo $help_meta_own_metafield_name ?>><span style="font-weight:bold;"><?php echo __('Define own field name','foodle-for-democracy-poll').':' ?></span><br>
                <input type="text" style="font-size:0.95em;<?php echo $foodle_own_field_style ?>" id="own-foodle-sel-<?php echo $foodle_meta_count ?>" name="own_meta_field[]" value="<?php echo $foodle_meta_field; ?>" placeholder="<?php echo __('Field key and name','foodle-for-democracy-poll') ?>" onkeyup="foodle_check_metafield_name(this);" <?php echo $required." ".$foodle_own_field_attr ?>>
              </label><br><br>
              <label <?php echo $help_meta_regexp_def ?>style="color:DarkGoldenRod;"><span style="font-weight:bold;"><?php echo __('Optional RegExp','foodle-for-democracy-poll').'...' ?></span><br>
                <input type="text" style="font-size:0.95em; border-color:DarkGoldenRod; <?php echo $foodle_regexp_style ?>" id="regexp-foodle-sel-<?php echo $foodle_meta_count ?>" name="regexp_field[]" value="<?php echo $foodle_regexp_field ?>" placeholder="<?php echo __('Regular Expression','foodle-for-democracy-poll') ?>" onkeyup="foodle_check_regexp_input(this);" onfocusin="if ( ( this.value.match(/^[/]/) == null ) && ( $(this).attr('readonly') != 'readonly' ) ) this.value='';" onfocusout="foodle_check_regexp_static($(this)); foodle_check_regexp($(this));" autocomplete="off" <?php echo $foodle_regexp_attr ?>>
                <br>&nbsp;&nbsp;&nbsp;--- = <?php echo $not_used; ?>
              </label>
            </td>
            <td style="padding: 10px 10px;">
              <label <?php echo $warning_meta_existing_metafield_name.$help_meta_existing_metafield_name; ?>><span style="font-weight:bold;"><?php echo __('Select existing field name','foodle-for-democracy-poll').':' ?></span><br>
                <select style="font-size:0.95em;" id="exist-foodle-sel-<?php echo $foodle_meta_count ?>"  name="existing_meta_field[]" size="1" onchange="foodle_manage_meta($(this));">
                  <option style="font-size:0.95em;" value="own-field" selected="selected"><?php echo __('Use own field name','foodle-for-democracy-poll')?></option>
                  <?php foreach ( $all_user_meta as $user_meta_key ) { ?>
                    <option style="font-size:0.95em;" value="<?php echo $user_meta_key;?>" <?php if ( $user_meta_key == $foodle_existing_field ) echo 'selected="selected"';?>><?php echo $user_meta_key; ?></option>
                  <?php } ?>
                </select>
              </label><br><br>
              <label <?php echo $help_meta_regexp_repl ?>style="color:DarkGoldenRod;"><span style="font-weight:bold;"><?php echo __('... and replacement','foodle-for-democracy-poll').':' ?></span><br>
                <input type="text" style="font-size:0.95em;
                  border-color:DarkGoldenRod; <?php echo $foodle_regrep_style ?>" id="regrep-foodle-sel-<?php echo $foodle_meta_count ?>" name="regrep_field[]" value="<?php echo $foodle_regrep_field ?>" placeholder="<?php echo __('Match replacement','foodle-for-democracy-poll') ?>" onkeyup="foodle_check_text_input(this);" onfocusin="if ( ( this.value.match(/^[a-zA-Z]/) == null ) && ( $(this).attr('readonly') != 'readonly' ) ) this.value='';" onfocusout="foodle_check_content($(this));" <?php echo $foodle_regrep_attr ?>>
                <br>&nbsp;&nbsp;&nbsp;--- = <?php echo __('eliminate match','foodle-for-democracy-poll'); ?>
              </label>
            </td>
            <td style="padding: 10px 10px;">
              <label <?php echo $help_meta_link_input ?>><span style="font-weight:bold;"><?php echo __('Link to existing ID','foodle-for-democracy-poll').':' ?></span><br>
                <input type="text" style="font-size:0.95em; <?php echo $foodle_link_meta_field_style ?>" id="link-foodle-sel-<?php echo $foodle_meta_count ?>"   name="link_meta_field[]" value="<?php echo $foodle_link_meta_field; ?>" placeholder="<?php echo __('Original meta field input ID','foodle-for-democracy-poll') ?>" onkeyup="foodle_check_text_input(this);" onfocusin="if ( ( this.value.match(/^[a-zA-Z]/) == null ) && ( $(this).attr('readonly') != 'readonly' ) ) this.value='';" onfocusout="foodle_check_content($(this));" <?php echo $foodle_link_meta_field_attr ?>>
                <br>&nbsp;&nbsp;&nbsp;--- = <?php echo $not_used; ?>
              </label><br><br>
              <label <?php echo $help_meta_poll_allow ?>><span style="font-weight:bold;"><?php echo __('Allow selection (category per answer column) in democracy poll edit','foodle-for-democracy-poll').':' ?></span><br>
                <label><input type="radio" style="margin:16px 4px 18px 0px; font-size:0.95em;" name="foodle_poll_use<?php echo $foodle_meta_count ?>" <?php echo $poll_use_yes_checked ?> value="yes" /><?php echo __('yes','foodle-for-democracy-poll') ?></label><label><input type="radio" style="margin:16px 4px 18px 18px; font-size:0.95em;" name="foodle_poll_use<?php echo $foodle_meta_count ?>" <?php echo $poll_use_no_checked ?> value="no" /><?php echo __('no','foodle-for-democracy-poll') ?></label>
              </label>
            </td>
            <td style="padding: 10px 10px;">
              <label <?php echo $help_meta_description ?>><span style="font-weight:bold;"><?php echo __('Input description','foodle-for-democracy-poll').':' ?></span><br>
                <textarea style="font-size:0.95em;" id="descr-foodle-sel-<?php echo $foodle_meta_count ?>" name="description[]" placeholder="<?php echo __('Description displayed under the meta field','foodle-for-democracy-poll') ?>" rows="4" cols="30" onkeyup="foodle_check_text_input(this);" onfocusin="if ( this.value.match(/^[a-zA-Z]/) == null ) this.value='';" onfocusout="foodle_check_content($(this));"><?php echo $foodle_description ?></textarea>
                <br>&nbsp;&nbsp;&nbsp;--- = <?php echo __('no description','foodle-for-democracy-poll'); ?>
              </label>
            </td>
            <td style="padding: 10px 10px;">
              <div style="display:block; padding: 10px;">
                <input <?php echo $help_meta_delete_row ?>type="button" style="padding:0px; border: 0px solid darkred; color:white; font-size:1em; width:25px; height:25px;<?php echo $foodle_del_button_style; ?> background-color:darkred; border-radius:25px;" id="del-foodle-sel-<?php echo $foodle_meta_count ?>" value="✘" onclick="foodle_remove_meta_row($(this));" <?php echo $foodle_del_button_attr; ?>><br><br>
                <input type="hidden" name="foodle_meta_line[]" value="<?php echo $foodle_meta_count ?>">
                <input <?php echo $help_meta_add_row ?>type="button" style="padding:5px 8px; border: 0px solid darkgreen; color:white; font-size:1em; width:25px; height:25px;<?php echo $foodle_add_button_style; ?> background-color:darkgreen; border-radius:25px;" id="add-foodle-sel-<?php echo $foodle_meta_count ?>" value="✛" onclick="foodle_add_meta_row($(this));" <?php echo $foodle_add_button_attr; ?>>
              </div>
            </td>
          </tr>
  <?php
  }
  ?>
        </tbody>
      </table>
    </figure>

    <script type="text/javascript">
      var $ = jQuery;
      var foodle_regexp_good = true;
      var foodle_regexp_error = '';
      var foodle_regexp_this;
      var foodle_scroll_store;
      var foodle_width_allow_unset = true;
      var plugin_url = '<?php echo plugin_dir_url(__FILE__); ?>';

      function foodle_activate_event_handler_to_store_meta_scrolltop() {
        $('.foodle-meta-handle').on('mousedown', function() {
          foodle_scroll_store = $('html').scrollTop();
        });
      }

      function foodle_activate_event_handler_to_set_meta_column_witdhs() {
        $('.foodle-meta-handle').on('mousedown', function() {
          $(".foodle-meta-table").css("width", $(".foodle-meta-table").outerWidth());
          $(".foodle-meta-table th, .foodle-meta-table td").each( function() {
            $(this).css("width", $(this).width() + 0.001);
          });
        });
        $('.foodle-meta-handle').on('mouseup', function() {
          if ( foodle_width_allow_unset )
            $(".foodle-meta-table, .foodle-meta-table th, .foodle-meta-table td").each( function() {
              $(this).css("width", '');
          });
        });
      }

      function foodle_unset_meta_column_witdhs() {
        $(".foodle-meta-table, .foodle-meta-table th, .foodle-meta-table td").each( function() {
          $(this).css("width", '');
        });
      }

      $('#foodle-meta-sortable').sortable({
        opacity: 0.75,
        revert: true,
        forcePlaceholderSize: true,
        forceHelperSize: true,
        handle: '.foodle-meta-handle',
        //axis: 'y',
        pull: 'clone',
        tolerance: 'pointer',
        delay: 150,
        scroll: true,
        scrollSensitivity: 10,
        scrollSpeed: 40,
        create: function(e, ui){
          foodle_activate_event_handler_to_store_meta_scrolltop();
          foodle_activate_event_handler_to_set_meta_column_witdhs();
        },
        start: function(e, ui){
          ui.placeholder.css('visibility', 'visible');
          ui.placeholder.children().css({'background-color':'AliceBlue','background-image':'url("' + plugin_url + 'img/foodle.png")','background-repeat':'no-repeat','background-position':'center','opacity':'0'}).fadeTo(660,0.4);
          ui.placeholder.height(ui.item.outerHeight());
          foodle_width_allow_unset = false;
        },
        activate: function(e, ui){
          $('html').scrollTop(foodle_scroll_store);
        },
        stop: function(e, ui){
          foodle_unset_meta_column_witdhs();
          foodle_width_allow_unset = true;
        }
      });



      function foodle_add_meta_row(foodle_this) {
        foodle_meta_last_row = foodle_this.closest('tr').prop('outerHTML');

        idtitle = 'foodle-meta-row-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        newcount = (foodle_meta_last_row.substring( posstart + idlength, posend)) * 1 + 1; // this value is valid for all further fields
        new_val = idtitle + newcount;
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'own-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'exist-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'link-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'descr-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'add-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'del-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'regexp-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'regrep-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        idtitle = 'foodle_poll_use'; // is a name, not an ID, but variable names are unchanged for convenience
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        const regex = new RegExp(old_val,'g'); // replace all occurrences
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(' checked="checked"', '');
        foodle_meta_last_row = foodle_meta_last_row.replace(regex, new_val); // all occurrences
        foodle_meta_last_row = foodle_meta_last_row.replace(new_val, new_val + '" checked="checked'); // just the first

        idtitle = 'foodle_meta_line[]" value="';
        idlength = idtitle.length; // characters
        posstart = foodle_meta_last_row.indexOf(idtitle);
        posend = foodle_meta_last_row.indexOf('"', posstart);
        old_val = foodle_meta_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_meta_last_row = foodle_meta_last_row.replace(old_val, new_val);

        $('.foodle-meta-row').last().after(foodle_meta_last_row);

        oldcount = newcount - 1;
        $('#foodle-meta-row-' + newcount).css('background-color','#dddddd').css('border-top-width','1px');

        $('#own-foodle-sel-' + newcount).removeAttr('readonly').css('cursor','').css('opacity','1.0').val('');
        $('#exist-foodle-sel-' + newcount).val('own-field');
        $('#link-foodle-sel-' + newcount).attr('readonly','readonly').css('cursor','not-allowed').css('opacity','0.4').val('---');
        $('#descr-foodle-sel-' + newcount).val('---');
        $('#regexp-foodle-sel-' + newcount).removeAttr('readonly').css('cursor','').css('opacity','1.0').val('---');
        $('#regrep-foodle-sel-' + newcount).removeAttr('readonly').css('cursor','').css('opacity','1.0').val('---');
        $('#add-foodle-sel-' + oldcount).attr('disabled','disabled').css('cursor','not-allowed').css('opacity','0.1');
        $('#del-foodle-sel-' + newcount).removeAttr('disabled').css('cursor','pointer').css('opacity','1.0');
        foodle_activate_tooltips(); // to catch the new row as well
        foodle_activate_event_handler_to_store_meta_scrolltop(); // to catch the new row as well
        foodle_activate_event_handler_to_set_meta_column_witdhs(); // to catch the new row as well
      }



      function foodle_remove_meta_row(foodle_this) {
        foodle_this.closest('tr').remove();
        $('.foodle-meta-row').last().children().last().children().first().children().last().removeAttr('disabled').css('opacity','1.0').css('cursor','pointer');
      }



      function foodle_manage_meta(foodle_this) {
        own_input_id = '#own' + foodle_this.attr('id').substring(5);
        link_input_id = '#link' + foodle_this.attr('id').substring(5);
        if ( foodle_this.val() == 'own-field' ) {
          $(own_input_id).removeAttr('readonly').css('cursor','').val('').css('opacity','1.0');
          $(link_input_id).attr('readonly','readonly').css('cursor','not-allowed').val('---').css('opacity','0.4');
        } else {
          $(own_input_id).attr('readonly','readonly').css('cursor','not-allowed').val('---').css('opacity','0.4');
          $(link_input_id).removeAttr('readonly').css('cursor','').val('---').css('opacity','1.0');
        }
      }



      function foodle_check_metafield_name(foodle_this) {
        if ( ( ! ( ( event.shiftKey ) && ( ( event.keyCode == 37 ) || ( event.keyCode == 38 ) || ( event.keyCode == 39 ) || ( event.keyCode == 40 ) ) ) ) && ( event.keyCode != 16 ) && ( event.keyCode != 17 ) && ( event.keyCode != 18 ) ) {
          foodle_save_curpos = foodle_this.selectionStart;
          foodle_save_selend = foodle_this.selectionEnd;
          foodle_save_value = foodle_this.value;

          // this is the very check - the rest is just there to smoothen the user input
          foodle_this.value=foodle_this.value.replace(/^[^a-zA-Z]/, '').replace(/[^a-zA-Z0-9.\-_ ]/g, '').replace(/\s+/g, ' ');

          foodle_len_diff = foodle_this.value.length - foodle_save_value.length;
          if ( foodle_this.value.length > foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos - 1;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          }
          if ( foodle_this.value.length < foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos + foodle_len_diff;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          } else {
            foodle_this.selectionStart = foodle_save_curpos - foodle_len_diff;
            foodle_this.selectionEnd = foodle_save_selend - foodle_len_diff;
          }
        }
      }



      function foodle_check_content(foodle_this) {
        $v = foodle_this.val();
        if ( $v.match(/^[a-zA-Z]/) == null ) { // must start with a letter
          foodle_this.val('---');
        }
      }



      function foodle_check_text_input(foodle_this) {
        if ( ( ! ( ( event.shiftKey ) && ( ( event.keyCode == 37 ) || ( event.keyCode == 38 ) || ( event.keyCode == 39 ) || ( event.keyCode == 40 ) ) ) ) && ( event.keyCode != 16 ) && ( event.keyCode != 17 ) && ( event.keyCode != 18 ) ) {
          foodle_save_curpos = foodle_this.selectionStart;
          foodle_save_selend = foodle_this.selectionEnd;
          foodle_save_value = foodle_this.value;

          // this is the very check - the rest is just there to smoothen the user input
          foodle_this.value=foodle_this.value.replace(/^[^a-zA-Z]/, '');

          foodle_len_diff = foodle_this.value.length - foodle_save_value.length;
          if ( foodle_this.value.length > foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos - 1;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          }
          if ( foodle_this.value.length < foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos + foodle_len_diff;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          } else {
            foodle_this.selectionStart = foodle_save_curpos - foodle_len_diff;
            foodle_this.selectionEnd = foodle_save_selend - foodle_len_diff;
          }
        }
      }



      function foodle_check_regexp_static(foodle_this) {
        $v = foodle_this.val();
        if ( $v.match(/^[/]/) == null ) { // must start with '/'
          foodle_this.val('---');
        }
      }



      function foodle_check_regexp_input(foodle_this) {
        if ( ( ! ( ( event.shiftKey ) && ( ( event.keyCode == 37 ) || ( event.keyCode == 38 ) || ( event.keyCode == 39 ) || ( event.keyCode == 40 ) ) ) ) && ( event.keyCode != 16 ) && ( event.keyCode != 17 ) && ( event.keyCode != 18 ) ) {
          foodle_save_curpos = foodle_this.selectionStart;
          foodle_save_selend = foodle_this.selectionEnd;
          foodle_save_value = foodle_this.value;

          // this is the very check - the rest is just there to smoothen the user input
          foodle_this.value=foodle_this.value.replace(/^[^/]/, '');

          foodle_len_diff = foodle_this.value.length - foodle_save_value.length;
          if ( foodle_this.value.length > foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos - 1;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          }
          if ( foodle_this.value.length < foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos + foodle_len_diff;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          } else {
            foodle_this.selectionStart = foodle_save_curpos - foodle_len_diff;
            foodle_this.selectionEnd = foodle_save_selend - foodle_len_diff;
          }
        }
      }



      function foodle_check_regexp(foodle_this) {
        foodle_regexp_good = true;
        foodle_regexp_error = '';
        foodle_regexp_value = foodle_this.val();
        if ( foodle_regexp_value == '---' ) return; // no Regular Expression
        foodle_regexp_first = foodle_regexp_value.indexOf('/');
        foodle_regexp_last = foodle_regexp_value.lastIndexOf('/');
        if ( foodle_regexp_first == foodle_regexp_last ) {
          foodle_regexp_this = foodle_this;
          foodle_regexp_good = false;
          foodle_regexp_error = 'SyntaxError: Missing pattern delimiter.';
        } else {
          if ( ( foodle_regexp_value != '---' ) && ( foodle_regexp_first == foodle_regexp_last ) ) {
            foodle_regexp_this = foodle_this;
            foodle_regexp_good = false;
          }
          else if ( foodle_this.attr('id').substring(0,6) == 'regexp' ) { // just to be sure
            foodle_regexp_pattern = foodle_regexp_value.substr(foodle_regexp_first + 1, foodle_regexp_last - foodle_regexp_first - 1);
            foodle_regexp_flags = foodle_regexp_value.substr(foodle_regexp_last + 1);
            try {
              var regex = new RegExp(foodle_regexp_pattern, foodle_regexp_flags);
              foodle_this.css('border-color', 'DarkGoldenRod');
              foodle_regexp_good = true;
            }
            catch (e) {
              foodle_regexp_this = foodle_this;
              foodle_regexp_good = false;
              foodle_regexp_error = e;
            }
          }
        }
      }



      function foodle_false_regexp_prevent_submit() {
        if ( foodle_regexp_good ) {
          return true;
        } else {
          regexp_content = foodle_regexp_this.val();
          foodle_regexp_this.css('border-color', 'red');

          foodle_regexp_this.focus();
          alert('<?php echo __('Invalid Regular Expression','foodle-for-democracy-poll') ?>!\n\n' + foodle_regexp_error);
          return false;
        }
      }



      $(document).ready(function() {
      $(window).keydown(function(event){
        if(event.keyCode == 13) {
          event.preventDefault();
          return false;
        }
        });
      });
    </script>
  <?php

  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" name="save_metafields_definition" value="'.__('Save metafields definition','foodle-for-democracy-poll').'..." onclick="return foodle_false_regexp_prevent_submit();"/></form></p>';
}



function foodle_metafield_defaults_sorting() {
  global $foodle_sorting;
  global $foodle_warnings_removed;
  global $foodle_help_tooltips;

  // Warnings and Help Tooltips
  $warning_sort_existing_metafield = ( ! $foodle_warnings_removed ) ? ' foodle_ttwarning="'.__('Warning:<br>Be careful when modifying content of unknown existing fields.<br>This may have unwanted effects and may harm WordPress or other plugins!','foodle-for-democracy-poll').'" ' : '';
  $warning_sort_metafield_error_dont_use = ( $foodle_help_tooltips ) ? ' foodle_ttwarning_dy="-63" foodle_ttwarning="'.__('This meta field is obviously not to be used!<br>Array content has been detected!','foodle-for-democracy-poll').'" ' : '';
  $warning_sort_metafield_error_disabled = ( $foodle_help_tooltips ) ? ' foodle_ttwarning_dy="-63"  foodle_ttwarning="'.__('This meta field is obviously not to be used!<br>It was disabled in the user profile!','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_name = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('The meta field name.<br><br><strong>Whether own or existing meta field:</strong><br>The meta field name will be displayed as field title in the user profile.<br>Sorting is always done in the same order as the fields are defined!','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_input = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('One meta field content found (number of occurrences).','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_input_replace = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Replace the existing input found (and all its occurrences) by a new input (= sub-category for the polls).<br>This may be used to normalize user inputs if not done through (or for to prepare) drop-down.<br><span style=\'color:darkred;\'>However, you must take extreme special care when modifying content of<br>such existing user meta fields as the results may not be foreseeable!</span>','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_not_used_yet = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('This meta field is currently not used.','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_sort = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('If selected:<br>The below list is used for meta field content (sub-categories) sorting only.<br>In this case, you may as well use substrings for sorting.','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_sort_drop_down = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('If selected:<br>The below list is used for meta field content (sub-categories) sorting and drop-down entry.<br>Users can only select data found in this list!<br>A Drop-Down will override Regular Expressions!','foodle-for-democracy-poll').'" ' : '';
  $help_sort_metafield_sort_list = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('This list defines the sorting for the sub-categories (meta field input).<br>One line per element is required.<br>For Drop-Down, leaving the first line empty will allow to leave the field empty<br>when a user wants to save his profile without such content.','foodle-for-democracy-poll').'" ' : '';

  echo '<p><form action="'.admin_url( 'admin.php?page=foodle-admin-page&tab=metafield-default-sorting' ).'" method="post">';
  ?>
    <figure class="foodle-block-table">
      <table class="foodle-sort-table">
        <tbody>
          <tr class="foodle-header-row">
            <th style="padding:0px 10px; text-align:center;"><br><?php echo __('Meta Field Name','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:0px 10px; text-align:center;" colspan="2"><br><?php echo __('Current Values<br>& Replacements','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:0px 10px; text-align:center;"><br><?php echo $foodle_sorting ?><br><br></th>
          </tr>
  <?php
  $users = get_users(array(
    'orderby'  => 'meta_value',
    'meta_key' => 'last_name', // just a habit ;-)
    'order'    => 'ASC'
  ));
  $foodle_user_metas = array();
  foreach((array)get_option('foodle_meta_fields') as $foodle_meta_field => $foodle_meta_field_data ) {
    foreach($users as $user) {
      update_meta_cache( 'user', array($user->ID) ); // get new data in case it was updated
      $foodle_user_meta = get_user_meta($user->ID, foodle_fieldname_to_meta_name($foodle_meta_field), true );
      if ( ( $foodle_user_meta != '' ) &&  ( ! is_array($foodle_user_meta) ) )
        $foodle_user_metas[$foodle_meta_field][$foodle_user_meta] = ( isset($foodle_user_metas[$foodle_meta_field][$foodle_user_meta]) ) ? $foodle_user_metas[$foodle_meta_field][$foodle_user_meta] + 1 : 1 ;
      else
        if ( is_array($foodle_user_meta) )
          $foodle_user_metas[$foodle_meta_field]['Error: don\'t use!'] = ( isset ($foodle_user_metas[$foodle_meta_field]['Error: don\'t use!']) ) ? $foodle_user_metas[$foodle_meta_field]['Error: don\'t use!'] + 1 : 1 ;
    }
    if ( isset($foodle_user_metas[$foodle_meta_field]) )
      ksort($foodle_user_metas[$foodle_meta_field]);
    else
      $foodle_user_metas[$foodle_meta_field] = array('•••not•used•yet•••' => 0);
  }
  $foodle_meta_defaults_sorting = array();
  $is_foodle_meta_defaults_sorting = true;
  if ( get_option('foodle_meta_defaults_sorting') ) {
    $foodle_meta_defaults_sorting = get_option('foodle_meta_defaults_sorting');
  } else {
    $is_foodle_meta_defaults_sorting = false;
  }
  $foodle_new_sub_category = __('Modify sub-category','foodle-for-democracy-poll');
  $foodle_one_line_per_item = __('One line per item (sorted)','foodle-for-democracy-poll');
  if ( count($foodle_user_metas) == 0 ) echo '<tr><td style="text-align:center;padding:10px; color:SteelBlue;" colspan="4">'.__('No categories to define sorting for, yet','foodle-for-democracy-poll').'!</td></tr>';
  foreach($foodle_user_metas as $foodle_user_meta => $foodle_user_meta_data ) {
    $foodle_number_of_lines = count($foodle_user_meta_data);
    $foodle_row_count = -1; // to get 0 as first iteration
    foreach ( $foodle_user_meta_data as $foodle_user_meta_input => $foodle_user_meta_count ) {
      $foodle_row_count += 1; // just count the rows
      $foodle_checked_list = 'checked="checked"';
      $foodle_checked_drop_down = '';
      $foodle_sortlist = '';
      if ( ( $is_foodle_meta_defaults_sorting) && ( isset($foodle_meta_defaults_sorting[$foodle_user_meta]['sorttype']) ) ) {
        if ( $foodle_meta_defaults_sorting[$foodle_user_meta]['sorttype'] == 'drop-down') {
          $foodle_checked_list = '';
          $foodle_checked_drop_down = 'checked="checked"';
        }
        $foodle_sortlist = str_replace('<br>','&#13;&#10;',$foodle_meta_defaults_sorting[$foodle_user_meta]['sortlist']);
        $foodle_number_of_sortitems = count(explode('<br>',$foodle_meta_defaults_sorting[$foodle_user_meta]['sortlist']));
      }
    ?>
          <tr class="foodle-sorting-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
    <?php
      if ( $foodle_row_count == 0 ):
    ?>
            <td <?php echo $help_sort_metafield_name ?>rowspan="<?php echo $foodle_number_of_lines ?>" style="padding:10px 10px; border-right: 1px solid #888888;">
              <?php echo str_replace('••', '', $foodle_user_meta) ?>
            </td>
    <?php
      endif;
      if ( ( $foodle_user_meta_input != '•••not•used•yet•••' ) && ( $foodle_user_meta_input != 'Error: don\'t use!' ) ):
    ?>
            <td style="padding:10px 10px;"><?php echo '<span'.$help_sort_metafield_input.'>'.str_replace(' ', '&nbsp;', $foodle_user_meta_input.'  ('.$foodle_user_meta_count.')').'</span>' ?></td>
            <td style="padding:10px 10px;"><label <?php echo $warning_sort_existing_metafield.$help_sort_metafield_input_replace ?>><?php echo sprintf(_n('Replace this entry by:', 'Replace all %d entries by:', $foodle_user_meta_count, 'foodle-for-democracy-poll'), $foodle_user_meta_count) ?><br><input type="text" name="subreplace[<?php echo $foodle_user_meta.']['.$foodle_user_meta_input ?>]" placeholder="<?php echo $foodle_new_sub_category ?>"></label></td>
    <?php
      elseif (  $foodle_user_meta_input == 'Error: don\'t use!' ):
    ?>
            <td colspan="2" style="padding:10px 10px; color:darkred;"><?php echo '<span'.$warning_sort_metafield_error_dont_use.'>'.__('Error: don\'t use (array content found)', 'foodle-for-democracy-poll').'!!!'.'</span>' ?></td>
    <?php
      else:
    ?>
            <td colspan="2" style="padding:10px 10px;"><?php echo '<span'.$help_sort_metafield_not_used_yet.'>'.__('Not used, yet', 'foodle-for-democracy-poll').'!'.'</span>' ?></td>
    <?php
      endif;
      if ( ( $foodle_row_count == 0 ) && ( $foodle_user_meta_input != 'Error: don\'t use!' ) ):
    ?>
            <td rowspan="<?php echo $foodle_number_of_lines ?>" style="padding:10px 10px; border-left: 1px solid #888888">
              <label <?php echo $help_sort_metafield_sort ?>><input type="radio" name="sorttype[<?php echo $foodle_user_meta ?>]" value="list" <?php echo $foodle_checked_list ?>><?php echo __('(Sub-)String List','foodle-for-democracy-poll')?></label>&nbsp;&nbsp;&nbsp;
              <label <?php echo $help_sort_metafield_sort_drop_down ?>><input type="radio" name="sorttype[<?php echo $foodle_user_meta ?>]" value="drop-down" <?php echo $foodle_checked_drop_down ?>><?php echo __('Drop-Down','foodle-for-democracy-poll'); ?></label><?php if ( ( isset(get_option('foodle_regex_main')[$foodle_user_meta]) ) && ( get_option('foodle_regex_main')[$foodle_user_meta] != '---' ) ): ?><br><span style="font-size:0.8em; color:darkred;"><?php echo str_replace(' ','&nbsp;', '('.__('Warning: A Regular Expression is set for this field','foodle-for-democracy-poll').')'); ?></span><?php endif ?><br><br>
              <label <?php echo $help_sort_metafield_sort_list ?>><?php echo __('Definition per sorted list (empty = not used)','foodle-for-democracy-poll') ?><br>
                <textarea name="sortlist[<?php echo $foodle_user_meta ?>]" rows="<?php echo ($foodle_number_of_sortitems + 1) ?>" cols="30" wrap="hard" placeholder="<?php echo $foodle_one_line_per_item ?>"><?php echo $foodle_sortlist ?></textarea>
              </label>
            </td>
    <?php
      elseif ( $foodle_user_meta_input == 'Error: don\'t use!' ):
    ?>
            <td rowspan="<?php echo $foodle_number_of_lines ?>" style="padding:10px 10px; border-left: 1px solid #888888; color:darkred;">
              <?php echo '<span'.$warning_sort_metafield_error_disabled.'>'.__('Error: Use has been disabled', 'foodle-for-democracy-poll').'!!!'.'</span>' ?>
            </td>
    <?php
      endif
    ?>
          </tr>
    <?php
    }
  }
  ?>
        </tbody>
      </table>
    </figure>
  <?php

  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" name="save_meta_defaults_sorting" value="'.__('Save defaults & sorting','foodle-for-democracy-poll').'..." onclick="return confirm(\''.__('Are you sure to submit the Replacements\nand the Defaults & Sorting data?\n\nThey will be applied immediately!','foodle-for-democracy-poll').'\');"/></form></p>';
}



function foodle_set_special_roles_users() {
  global $wp_roles;
  global $foodle_help_tooltips;

  $help_special_role_user = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Selection of roles and users.<br> A \'No selection\' entry will delete the row upon saving.','foodle-for-democracy-poll').'" ' : '';
  $help_special_capabilities = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('<strong><u>Definition of capabilities:</u></strong><br><strong>Mark:</strong> This role or user and his votes will be marked when found in a poll results list. It will,<br>however, not be considered in the category statistics per answer, nor for \'maxcount\'.<br><strong>Special View:</strong> This role or user can view (and execute) hidden content (like reminder activity).<br><strong>No Reminders:</strong> This role or user will receive no email reminders for poll votes.<br>(The latter may be typical for the administrator and a few other registered users.<br>&nbsp;Reason: This plugin would remind all registered users that didn\'t vote so far in a poll,<br>&nbsp;except those referred to herewith!)','foodle-for-democracy-poll').'" ' : '';
  $help_special_delete_row = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Delete this Foodle capabilities row.','foodle-for-democracy-poll').'" ' : '';
  $help_special_add_row = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Add a new Foodle capabilities row.','foodle-for-democracy-poll').'" ' : '';
  $help_dragging_su_rows = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('You may drag each row to where you want in order to change the row order.<br>This will become active <strong>upon saving</strong>.<br>However, this is a rather cosmetic, not a functional option.','foodle-for-democracy-poll').'"' : '';

  $users = get_users(array(
    'orderby'  => 'meta_value',
    'meta_key' => 'last_name', // just a habit ;-)
    'order'    => 'ASC'
  ));

  echo '<p><form action="'.admin_url( 'admin.php?page=foodle-admin-page&tab=special-roles-users' ).'" method="post">';
  ?>
    <figure class="foodle-block-table">
      <table unselectable="on" class="foodle-special-roles-table unselectable">
        <thead>
          <tr class="foodle-header-row">
            <th style="padding:0px 10px; text-align:center;"><div style="opacity:0.4; margin:auto; width:18px; height:16px; background-size:contain; background-image: url('<?php echo plugin_dir_url(__FILE__).'img/move_sm.png'; ?>');" <?php echo $help_dragging_su_rows ?>></div></th>
            <th style="padding:0px 10px; text-align:center;"><br><?php echo __('Select Roles & Users','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:0px 10px; text-align:center;"><br><?php echo __('Set Foodle Capabilities','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:0px 10px; text-align:center;">&nbsp;</th> <!-- the column with the buttons -->
          </tr>
        </thead>
        <tbody id="foodle-special-roles-sortable">
  <?php
  if ( ( get_option('foodle_special_functions') ) && ( count(get_option('foodle_special_functions')) != 0 ) ) {
    $foodle_special_functions = get_option('foodle_special_functions');
  } else {
    $foodle_special_functions['no_selection'] = array('');
  }

  $foodle_number_of_rows = count($foodle_special_functions);
  $foodle_meta_count = -1; // in order to have the first iteration see 0
  foreach ( $foodle_special_functions as $selected_user_role => $foodle_capabilities ) {
    $foodle_meta_count += 1;
    if ($foodle_meta_count == 0) {
      $foodle_del_button_style = 'opacity:0.1; cursor:not-allowed;';
      $foodle_del_button_attr = 'disabled="disabled"';
      if ( $foodle_number_of_rows == 1 ) { // just one row existing
        $foodle_add_button_style = 'opacity:1.0; cursor:pointer;';
        $foodle_add_button_attr = '';
      } else {
        $foodle_add_button_style = 'opacity:0.1; cursor:not-allowed;';
        $foodle_add_button_attr = 'disabled="disabled"';
      }
    } else {
      $foodle_del_button_style = 'opacity:1.0; cursor:pointer;';
      $foodle_del_button_attr = '';
      if ( $foodle_meta_count + 1 == $foodle_number_of_rows ) { // the last row
        $foodle_add_button_style = 'opacity:1.0; cursor:pointer;';
        $foodle_add_button_attr = '';
      } else {
        $foodle_add_button_style = 'opacity:0.1; cursor:not-allowed;';
        $foodle_add_button_attr = 'disabled="disabled"';
      }
    }
  ?>
          <tr class="foodle-functions-row" style="border-top: 1px solid #888888; background-color:#dddddd;" id="foodle-functions-row-<?php echo $foodle_meta_count ?>">
            <td style="padding: 10px 10px;">
              <div class="foodle-special-roles-handle" style="cursor:move; margin:auto; width:18px; height:16px; background-size:contain; background-image: url('<?php echo plugin_dir_url(__FILE__).'img/move_sm.png'; ?>');" <?php echo $help_dragging_su_rows ?>></div>
            </td>
            <td style="padding:10px 10px;">
              <label <?php echo $help_special_role_user ?>><span style="font-weight:bold;"><?php echo __('Role / User','foodle-for-democracy-poll').':' ?></span><br>
                <select style="font-size:0.95em;" id="roles-users-sel-<?php echo $foodle_meta_count ?>"  name="roles_users_field[]" size="1">
                  <optgroup label ="<?php echo __('Nothing','foodle-for-democracy-poll') ?>">
                    <option style="font-size:0.95em;" value="no_selection" <?php if ( $selected_user_role == 'no_selection' ) echo 'selected="selected"';?>><?php echo __('No selection','foodle-for-democracy-poll') ?></option>
                  </optgroup>
                  <optgroup label ="<?php echo __('Roles','foodle-for-democracy-poll') ?>">
                    <?php   foreach ( $wp_roles->role_names as $foodle_role_slug => $foodle_role_name) { ?>
                      <option style="font-size:0.95em;" value="<?php echo $foodle_role_slug;?>" <?php if ( $selected_user_role == $foodle_role_slug ) echo 'selected="selected"';?>><?php echo $foodle_role_name; ?></option>
                    <?php } ?>
                  </optgroup>
                  <optgroup label ="<?php echo __('Users','foodle-for-democracy-poll') ?>">
                    <?php foreach ( $users as $user ) {
                            $foodle_user_id = $user->ID;
                            $foodle_user_info = get_userdata($foodle_user_id);
                            $foodle_user = $foodle_user_info->user_login;
                           ?>
                      <option style="font-size:0.95em;" value="<?php echo $foodle_user_id;?>" <?php if ( $selected_user_role == $foodle_user_id ) echo 'selected="selected"';?>><?php echo $foodle_user; ?></option>
                    <?php } ?>
                  </optgroup>
                </select>
              </label>
            </td>
            <td style="padding:10px 10px;">
              <span  <?php echo $help_special_capabilities ?>>
                <span style="font-weight:bold;"><?php echo __('Capabilities','foodle-for-democracy-poll').':' ?></span><br>
                <label>
                  <input type ="checkbox" style="font-size:0.95em;" id="check-mark-<?php echo $foodle_meta_count ?>" name="capabilities<?php echo $foodle_meta_count ?>[]" value="mark" <?php if ( in_array('mark', $foodle_capabilities ) ) echo 'checked="checked"';?>/><?php echo __('Mark','foodle-for-democracy-poll') ?>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                </label>
                <label>
                  <input type ="checkbox" style="font-size:0.95em;" id="check-view-<?php echo $foodle_meta_count ?>" name="capabilities<?php echo $foodle_meta_count ?>[]" value="view" <?php if ( in_array('view', $foodle_capabilities ) ) echo 'checked="checked"';?>/><?php echo __('Special View','foodle-for-democracy-poll') ?>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                </label>
                <label>
                  <input type ="checkbox" style="font-size:0.95em;" id="check-no-remind-<?php echo $foodle_meta_count ?>" name="capabilities<?php echo $foodle_meta_count ?>[]" value="no-remind" <?php if ( in_array('no-remind', $foodle_capabilities ) ) echo 'checked="checked"';?>/><?php echo __('No Reminders','foodle-for-democracy-poll') ?>
                </label>
              </span>
            </td>
            <td style="padding:0px 10px;">
              <div style="display:block; padding:10px;">
                <input <?php echo $help_special_delete_row ?>type="button" style="padding:0px; border: 0px solid darkred; color:white; font-size:1em; width:25px; height:25px;<?php echo $foodle_del_button_style; ?> background-color:darkred; border-radius:25px;" id="del-foodle-sel-<?php echo $foodle_meta_count ?>" value="✘" onclick="foodle_remove_functions_row($(this));" <?php echo $foodle_del_button_attr; ?>><br><br>
                <input type="hidden" name="foodle_functions_line[]" value="<?php echo $foodle_meta_count ?>">
                <input <?php echo $help_special_add_row ?>type="button" style="padding:5px 8px; border: 0px solid darkgreen; color:white; font-size:1em; width:25px; height:25px;<?php echo $foodle_add_button_style; ?> background-color:darkgreen; border-radius:25px;" id="add-foodle-sel-<?php echo $foodle_meta_count ?>" value="✛" onclick="foodle_add_functions_row($(this));" <?php echo $foodle_add_button_attr; ?>>
              </div>
            </td>
          </tr>
  <?php
  }
  ?>
        </tbody>
      </table>
    </figure>

    <script type="text/javascript">
      var $ = jQuery;
      var foodle_scroll_store;
      var foodle_width_allow_unset = true;
      var plugin_url = '<?php echo plugin_dir_url(__FILE__); ?>';

      function foodle_activate_event_handler_to_store_special_roles_scrolltop() {
        $('.foodle-special-roles-handle').on('mousedown', function() {
          foodle_scroll_store = $('html').scrollTop();
        });
      }

      function foodle_activate_event_handler_to_set_special_roles_column_witdhs() {
        $('.foodle-special-roles-handle').on('mousedown', function() {
          $(".foodle-special-roles-table").css("width", $(".foodle-special-roles-table").outerWidth());
          $(".foodle-special-roles-table th, .foodle-special-roles-table td").each( function() {
            $(this).css("width", $(this).width() + 0.001);
          });
        });
        $('.foodle-special-roles-handle').on('mouseup', function() {
          if ( foodle_width_allow_unset )
            $(".foodle-special-roles-table, .foodle-special-roles-table th, .foodle-special-roles-table td").each( function() {
              $(this).css("width", '');
          });
        });
      }

      function foodle_unset_special_roles_column_witdhs() {
        $(".foodle-special-roles-table, .foodle-special-roles-table th, .foodle-special-roles-table td").each( function() {
          $(this).css("width", '');
        });
      }

      $('#foodle-special-roles-sortable').sortable({
        opacity: 0.66,
        revert: true,
        forcePlaceholderSize: true,
        forceHelperSize: true,
        handle: '.foodle-special-roles-handle',
        //axis: 'y',
        pull: 'clone',
        tolerance: 'pointer',
        delay: 150,
        scroll: true,
        scrollSensitivity: 10,
        scrollSpeed: 40,
        create: function(e, ui){
          foodle_activate_event_handler_to_store_special_roles_scrolltop();
          foodle_activate_event_handler_to_set_special_roles_column_witdhs();
        },
        start: function(e, ui){
          ui.placeholder.css('visibility', 'visible');
          ui.placeholder.children().css({'background-color':'AliceBlue','background-image':'url("' + plugin_url + 'img/foodle.png")','background-repeat':'no-repeat','background-position':'center','opacity':'0'}).fadeTo(660,0.4);
          ui.placeholder.height(ui.item.outerHeight());
          foodle_width_allow_unset = false;
        },
        activate: function(e, ui){
          $('html').scrollTop(foodle_scroll_store);
        },
        stop: function(e, ui){
          foodle_unset_special_roles_column_witdhs();
          foodle_width_allow_unset = true;
        }
      });



      function foodle_add_functions_row(foodle_this) {
        foodle_functions_last_row = foodle_this.closest('tr').prop('outerHTML');

        idtitle = 'roles-users-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        newcount = (foodle_functions_last_row.substring( posstart + idlength, posend)) * 1 + 1; // this value is valid for all further fields
        new_val = idtitle + newcount;
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        idtitle = 'capabilities';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('[', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        const regex = new RegExp(old_val,'g'); // replace all occurrences
        new_val = idtitle + newcount;
        foodle_functions_last_row = foodle_functions_last_row.replace(regex, new_val); // all occurrences

        idtitle = 'check-mark-';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        new_val = idtitle + newcount;
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        idtitle = 'check-view-';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        idtitle = 'check-no-remind-';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        idtitle = 'add-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        idtitle = 'del-foodle-sel-';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        idtitle = 'foodle_functions_line[]" value="';
        idlength = idtitle.length; // characters
        posstart = foodle_functions_last_row.indexOf(idtitle);
        posend = foodle_functions_last_row.indexOf('"', posstart);
        old_val = foodle_functions_last_row.substring(posstart, posend);
        new_val = idtitle + newcount; // newcount defined before
        foodle_functions_last_row = foodle_functions_last_row.replace(old_val, new_val);

        $('.foodle-functions-row').last().after(foodle_functions_last_row);

        oldcount = newcount - 1;

        $('#roles-users-sel-' + newcount).children().children().removeAttr('selected');
        $('#roles-users-sel-' + newcount).children().first().children().first().attr('selected','selected');

        $('#check-mark-' + newcount).removeAttr('checked');
        $('#check-view-' + newcount).removeAttr('checked');
        $('#check-no-remind-' + newcount).removeAttr('checked');

        $('#add-foodle-sel-' + oldcount).attr('disabled','disabled').css('cursor','not-allowed').css('opacity','0.1');
        $('#del-foodle-sel-' + newcount).removeAttr('disabled').css('cursor','pointer').css('opacity','1.0');
        foodle_activate_tooltips(); // to catch the new row as well
        foodle_activate_event_handler_to_store_special_roles_scrolltop(); // to catch the new row as well
        foodle_activate_event_handler_to_set_special_roles_column_witdhs(); // to catch the new row as well
      }



      function foodle_remove_functions_row(foodle_this) {
        foodle_this.closest('tr').remove();
        $('.foodle-functions-row').last().children().last().children().first().children().last().removeAttr('disabled').css('opacity','1.0').css('cursor','pointer');
      }



      $(document).ready(function() {
        $(window).keydown(function(event) {
          if(event.keyCode == 13) {
            event.preventDefault();
            return false;
          }
        });
      });
    </script>
  <?php

  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" name="save_special_roles_users" value="'.__('Save special users setting','foodle-for-democracy-poll').'..."/></form></p>';
}



function foodle_edit_email() {

  if (!get_option("foodle_email_content") )
    echo '<div style="color:#500404;">('.__('The eMail content has not been set, yet. You will not be able to send reminder emails to late poll users!','foodle-for-democracy-poll').')</div><br />';
  else
    echo '<div style="color:#045004;">('.__('The eMail content is ready to use. You may initiate reminder emails to late poll users!','foodle-for-democracy-poll').')</div><br />';

  echo '<p><table><tr><th>'.__('eMail Content','foodle-for-democracy-poll').'</th></tr></table></p>';

  $settings = array(
    'wpautop' => false,
    'media_buttons' => true,
    'default_editor' => '',
    'drag_drop_upload' => true,
    'textarea_rows' => 30,
    'tabindex' => 5,
    'editor_css' => '',
    'editor_class' => 'foodle-email-editor',
    'teeny' => false,
    'tinymce' => true,
    'quicktags' => true
  );
  $foodle_email_content = '';
  if ( get_option('foodle_email_content') ) $foodle_email_content = html_entity_decode(get_option('foodle_email_content'));
  echo '<form name="foodle_email_editor" action="'.admin_url( 'admin.php?page=foodle-admin-page&tab=edit-email' ).'" method="post">';
  echo '<div style="width:90%;">';
  wp_editor($foodle_email_content, 'foodle_email_content', $settings);
  echo '</div>';
  echo '<br><strong>'.__('Available placeholders','foodle-for-democracy-poll').':</strong><br>
  '.__('Poll','foodle-for-democracy-poll').':&nbsp;&nbsp;{foodle-title}&nbsp;&nbsp;{poll-ID}&nbsp;&nbsp;{poll-title}&nbsp;&nbsp;{poll-start}&nbsp;&nbsp;{poll-end}&nbsp;&nbsp;{users-voted}&nbsp;&nbsp;{poll-url}&nbsp;&nbsp;{poll-link}<br>
  '.__('User','foodle-for-democracy-poll').':&nbsp;&nbsp;{user-login}&nbsp;&nbsp;{first-name}&nbsp;&nbsp;{last-name}&nbsp;&nbsp;{display-name}&nbsp;&nbsp;{nickname}&nbsp;&nbsp;{user-nicename}<br>
  '.__('Admin','foodle-for-democracy-poll').':&nbsp;&nbsp;{firstname-trigger}&nbsp;&nbsp;{lastname-trigger}&nbsp;&nbsp;{password-forgot-url}&nbsp;&nbsp;{password-forgot-link}';
  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" value="'.__('Update email content...','foodle-for-democracy-poll').'" /></form>';
}



function foodle_settings() {
  global $foodle_title;
  global $foodle_admin_menu_color;
  global $foodle_warnings_removed;
  global $foodle_help_tooltips;
  global $foodle_results_text_default;
  global $foodle_results_text;

  $help_settings_foodle_title = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('The \'Foodle Title\' will be used and displayed throughout this plugin<br>in several places including the user profile area.','foodle-for-democracy-poll').'" ' : '';
  $help_settings_admin_menu_color = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('In order to identify this plugin and Democracy Poll easier in the admin menu<br> and/or admin toolbar, you may chose any convenient highlighting color.<br>An invalid color name/value will retain the original color setting.<br>Default: #8CBD5A.','foodle-for-democracy-poll').'" ' : '';
  $help_settings_page_post_exclusion = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('With this comma-separated list of IDs, you may determine all pages and/or posts where<br>the Foodle table\'s interactive AJAX shall be switched off. This could become necessary<br>to avoid conflicts, e.g. when more than one Foodle shortcode is present.<br>(Content is being validated during input)','foodle-for-democracy-poll').'" ' : '';
  $help_settings_vote_expiry = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('When checked, a regular task will update the poll logs database to avoid vote expiry.<br>If not, Democracy Poll will not use any votes older than one year for re-voting.<br>In such case, the user can vote again in polls still active and his former vote<br>will remain visible in additon.','foodle-for-democracy-poll').'" ' : '';
  $help_settings_disable_warnings = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('When checked, warning tooltips will be disabled.','foodle-for-democracy-poll').'" ' : '';
  $help_settings_enable_help = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('When checked, Help-Tooltips (like this one) will be activated.','foodle-for-democracy-poll').'" ' : '';
  $help_settings_results_text = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('Define the text to be displayed above the Foodle table.<br>Three dashes (\'---\') means: no headline.','foodle-for-democracy-poll').'" ' : '';

  $foodle_vote_expiry_checked = '';
  if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['remove_vote_expiry']) ) ) {
    $foodle_vote_expiry_checked = ( get_option('foodle_settings')['remove_vote_expiry'] ) ? 'checked="checked"' : '';
  }

  $foodle_page_post_exclusion = '';
  if ( ( get_option('foodle_settings') ) && ( isset(get_option('foodle_settings')['foodle_page_post_exclusion']) ) )
    if ( get_option('foodle_settings')['foodle_page_post_exclusion'] != '' ) $foodle_page_post_exclusion = implode(',', get_option('foodle_settings')['foodle_page_post_exclusion']);

  $foodle_timestamp = ( wp_next_scheduled( 'foodle_cron_hook' ) ) ?  date_i18n('j. F Y', wp_next_scheduled( 'foodle_cron_hook' )) : __('Not scheduled','foodle-for-democracy-poll');

  $foodle_warnings_removed_checked = ( $foodle_warnings_removed ) ? 'checked="checked"' : '';

  $foodle_help_tooltips_checked = ( $foodle_help_tooltips ) ? 'checked="checked"' : '';

  echo '<p><form action="'.admin_url( 'admin.php?page=foodle-admin-page&tab=foodle-settings' ).'" method="post">';
  ?>
    <figure class="foodle-block-table">
      <table class="foodle-settings-table">
        <tbody>
          <tr class="foodle-header-row">
            <th style="padding:0px 10px; text-align:center;"><br><?php echo __('Variable','foodle-for-democracy-poll') ?><br><br></th>
            <th style="padding:0px 10px; text-align:center;"><br><?php echo __('Value','foodle-for-democracy-poll') ?><br><br></th>
          </tr>
          <tr <?php echo $help_settings_foodle_title ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Foodle Title','foodle-for-democracy-poll').':' ?>
              </span><br><?php echo '('.__('This title is used throughout the plugin incl. the profile pages','foodle-for-democracy-poll').')' ?>
            </td>
            <td style="padding:10px 10px;">
                <input type="text" style="font-size:0.95em;" id="foodle-title" name="foodle_title" value="<?php echo $foodle_title; ?>" placeholder="Foodle"/>
            </td>
          </tr>
          <tr <?php echo $help_settings_results_text ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Foodle Table Headline','foodle-for-democracy-poll').':' ?>
              </span><br><?php echo '('.__('This text will be displayed above the Foodle table, \'---\' = no headline','foodle-for-democracy-poll').')' ?>
            </td>
            <td style="padding:10px 10px;">
                <input type="text" style="font-size:0.95em;" id="foodle-title" name="foodle_results_text" value="<?php echo $foodle_results_text; ?>" size="50" placeholder="<? echo $foodle_results_text_default ?>"/>
            </td>
          </tr>
          <tr <?php echo $help_settings_admin_menu_color ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Democracy Poll & Foodle Admin Menu Colors','foodle-for-democracy-poll').':' ?>
              </span><br><?php echo '('.__('Just meant as a smooth highlighting of these two plugin links','foodle-for-democracy-poll').')' ?>
            </td>
            <td style="padding:10px 10px;">
                <input type="text" style="font-size:0.95em;" id="admin-menu-color" name="foodle_admin_menu_color" value="<?php echo $foodle_admin_menu_color; ?>" placeholder="#8CBD5A"/>
            </td>
          </tr>
          <tr <?php echo $help_settings_page_post_exclusion ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Page and Post IDs with Foodle\'s interactive AJAX switched off','foodle-for-democracy-poll').':' ?>
              </span><br><?php echo '('.__('May be necessary to avoid conflicts in certain cases','foodle-for-democracy-poll').')' ?>
            </td>
            <td style="padding:10px 10px;">
                <input type="text" style="font-size:0.95em;" id="admin-menu-color" name="foodle_page_post_exclusion" value="<?php echo $foodle_page_post_exclusion; ?>" size="50" placeholder="<?php echo __('Comma-separated list','foodle-for-democracy-poll') ?>" onkeyup="foodle_check_post_list(this);"/>
            </td>
          </tr>
          <tr <?php echo $help_settings_vote_expiry ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Remove Democracy Poll votes\' expiring regularly','foodle-for-democracy-poll').':' ?>
              </span><br><?php echo '('.__('Typically one year from each individual vote','foodle-for-democracy-poll').')' ?>
            </td>
            <td style="padding:10px 10px;">
                <label><input type="checkbox" style="font-size:0.95em;" id="remove-vote-expiry" name="remove_vote_expiry" <?php echo $foodle_vote_expiry_checked; ?>  /> <?php echo str_replace(' ', '&nbsp;', __('Next check','foodle-for-democracy-poll').': '.$foodle_timestamp) ?></label>
            </td>
          </tr>
          <tr <?php echo $help_settings_disable_warnings ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Don\'t display the warnings any more','foodle-for-democracy-poll').':' ?>
              </span>
            </td>
            <td style="padding:10px 10px;">
                <label><input type="checkbox" style="font-size:0.95em;" id="remove-warnings" name="remove-warnings" <?php echo $foodle_warnings_removed_checked; ?>  /> <?php echo __('no warnings','foodle-for-democracy-poll') ?></label>
            </td>
          </tr>
          <tr <?php echo $help_settings_enable_help ?>class="foodle-settings-row" style="border-top: 1px solid #888888; background-color:#dddddd;">
            <td style="padding:10px 10px;">
              <span style="font-weight:bold;">
              <?php echo __('Switch on Help-Tooltips','foodle-for-democracy-poll').':' ?>
              </span>
            </td>
            <td style="padding:10px 10px;">
                <label><input type="checkbox" style="font-size:0.95em;" id="help-tooltips" name="help-tooltips" <?php echo $foodle_help_tooltips_checked; ?>  /> <?php echo __('Help-Tooltips','foodle-for-democracy-poll') ?></label>
            </td>
          </tr>
        </tbody>
      </table>
    </figure>

    <script type="text/javascript">
      var $ = jQuery;



      function foodle_check_post_list(foodle_this) {
        if ( ( ! ( ( event.shiftKey ) && ( ( event.keyCode == 37 ) || ( event.keyCode == 38 ) || ( event.keyCode == 39 ) || ( event.keyCode == 40 ) ) ) ) && ( event.keyCode != 16 ) && ( event.keyCode != 17 ) && ( event.keyCode != 18 ) ) {
          foodle_save_curpos = foodle_this.selectionStart;
          foodle_save_selend = foodle_this.selectionEnd;
          foodle_save_value = foodle_this.value;

          // this is the very check - the rest is just there to smoothen the user input
          foodle_this.value=foodle_this.value.replace(/[^0-9,]/g, '');

          foodle_len_diff = foodle_this.value.length - foodle_save_value.length;
          if ( foodle_this.value.length > foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos - 1;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          }
          if ( foodle_this.value.length < foodle_save_value.length ) {
            foodle_this.selectionStart = foodle_save_curpos + foodle_len_diff;
            foodle_this.selectionEnd = foodle_this.selectionStart;
          } else {
            foodle_this.selectionStart = foodle_save_curpos - foodle_len_diff;
            foodle_this.selectionEnd = foodle_save_selend - foodle_len_diff;
          }
        }
      }
    </script>
  <?php

  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" name="save_foodle_settings" value="'.__('Save Foodle settings','foodle-for-democracy-poll').'..."/></form></p>';
}



function foodle_tips() {
  global $foodle_like_me_1;
  global $foodle_like_me_2;
  global $foodle_help_tooltips;

  $switch_on_off = __('Switch On/Off','foodle-for-democracy-poll');
  $shortcode_parameters = __('Shortcode Parameters','foodle-for-democracy-poll');
  $explanations = __('Explanations','foodle-for-democracy-poll');
  $the_shortcode_slug = __('The shortcode slug','foodle-for-democracy-poll').'.';
  $help_tips_edit_pic = ( $foodle_help_tooltips ) ? ' foodle_tooltip="'.__('The Democracy Poll edit page enhanced with a few Foodle features','foodle-for-democracy-poll').'." ' : '';
  ?>
  <p class="foodle-headline"><?php echo __('Some General Considerations','foodle-for-democracy-poll') ?></p>

  <p class="foodle-text"><?php echo __('The basic idea for this plugin was developed when the need for polls arrived for the choir I have the pleasure to be part of: <a target="_blank" href="https://voicesunlimited.de">Voices Unlimited</a>.','foodle-for-democracy-poll') ?></p>

  <p class="foodle-text">
    <?php echo __('This plugin can be used by any organization, association, union, society, club or group of people when polls shall be executed and results be displayed in a structured way, e.g.:
      <ul class="foodle-ul">
        <li>For this or that choir rehearsal or planned public event to be scheduled, would sufficient Sopranos 1 or Tenors 2 be available?</li>
        <li>Or do the choir members support this or that idea and from which sub-group do they come from?</li>
        <li>Or what could be winning ideas for the next concert stage dress (Democracy Poll with answers to be brought by the voters and be displayed for everyone easy to understand) and would this spread nicely in color on stage, considering the choir\'s stage order?</li>
        <li>...you will surely translate all this into your own environment and your own needs...</li>
      </ul>','foodle-for-democracy-poll')
    ?>
  </p>

  <p class="foodle-text"><?php echo __('<strong>Main feature</strong> is the Foodle shortcode, which allows to display poll results in a configurable table format. Alongside the Democracy Poll shortcode, it will display the results interactively by use of AJAX communication. However, it can as well be used independently to just display poll results. Together with the WordPress comments function in pages or posts, this can be considered a good tool for online interaction with your registered (!) users.','foodle-for-democracy-poll') ?></p>

  <p class="foodle-text"><?php echo __('<strong>Warning! Democracy Poll\'s IP storing option must be activated in order for Foodle to work.</strong> By this, all votes are stored in the database for Foodle to work with. You may select to avoid IP storage in each individual poll in order to avoid failures when voters share the same IP address, e.g. when voting in the same WLAN environment.','foodle-for-democracy-poll') ?></p>

  <p class="foodle-text">
    <?php echo __('<strong>The list of Foodle features comprises:</strong>
      <ul class="foodle-ul">
        <li>The interactive poll results table (shortcode)</li>
        <li>This shortcode only makes sense and therefore only works for registered users (else: no display) and registered voters (else: message)</li>
        <li>All other functions work regardless this necessary restriction</li>
        <li>Several Democracy Poll shortcodes on one page/post (like before)</li>
        <li>Full shortcode flexibility: Several Foodle shortcodes - even for the same Democracy Poll id with differing parameters - on one page/post</li>
        <li>Email reminders for users who are late to vote</li>
        <li>An unlimited number of user meta fields to be defined and used (user profile)</li>
        <li>Manual drag-n-drop fields sorting</li>
        <li>Careful (!) selection and use of existing user meta fields</li>
        <li>Manual entry or drop-down entry in the user profile</li>
        <li>Automated filling of user meta fields by use of Regular Expressions</li>
        <li>Easy overview and control over past user entries (normalization)</li>
        <li>Flexible sorting definitions</li>
        <li>Definition of role & user plugin-internal capabilities</li>
        <li>Dashboard widget (and shortcode) to provide statistics and check for logical errors in democracy_q, democracy_a and democracy_log databases</li>
        <li>A shortcode to link to the Foodle page/post</li>
        <li>A shortcode to display content based on roles</li>
        <li>A free name/title definition for Foodle, e.g. for the user profile and the email</li>
        <li>Changeable highlighting of Foodle & Democracy Poll in the admin menu and admin toolbar</li>
        <li>You can define a list of page/post IDs where the AJAX interactivitiy is switched off.</li>
        <li>Switchable interactive explanations for all functions (\'Help-Tooltips\')</li>
        <li>Warning-Tooltips can be disabled</li>
        <li>Help-Tooltips and Warning-Tooltips are available anywhere for the administrator (front-end and back-end)</li>
        <li>A number of Foodle CSS classes to have the design control</li>
      </ul>','foodle-for-democracy-poll')
    ?>
  </p>

  <p class="foodle-text">
    <?php echo __('<strong>In addition to that, functionality is added to Democracy Poll:</strong>
      <ul class="foodle-ul">
        <li>In multiple answers polls, you can combine checkboxes and radio buttons</li>
        <li>In each poll edit page, you may select from the available categories (user meta fields) for display</li>
        <li>In each poll edit page, you may define the (main) category to be used for the category column</li>
        <li>In each poll edit page, you may define the related text of its sorting button</li>
        <li>In each poll edit page, you may decide whether to display an AJAX refresh button in the results displays (voted and/or not voted)</li>
        <li>In each poll edit page, you may decide whether to have marked users to be counted (columns sums and first-come/first-serve answers)</li>
        <li>In each poll edit page, you may decide whether to delete the IP adresses for this poll upon vote in order to preserve full functionality even when voters share the same IP address.<br><strong>Reminder:</strong> Democracy Poll\'s logging and IP storing option must remain activated in order for Foodle to work!</li>
        <li>In the graphical poll results, a text will indicate the voter\'s choice(s)</li>
        <li>You can choose to avoid the vote expiry on polls lasting longer than 12 months</li>
        <li>The use of shortcodes is now possible inside the Democracy Poll\'s own text field located underneath the very poll</li>
        <li>A shortcode to disable the display of certain Foodle IDs in an archive listing (i.e. by enclosing the related Democracy Poll archive shortcode)</li>
        <li>A number of Democracy CSS classes to have the design control</li>
      </ul>','foodle-for-democracy-poll')
    ?>
  </p>

  <p class="foodle-text"><?php echo __('You will probably - at least in the beginning - want to switch-on the \'Help-Tooltips\' (Foodle Settings)<br>to see explanations for every function throughout these tabs.<br>And don\'t hesitate to <a href="mailto:plugins@finkenberger.net?subject=Foodle%20Feature-Proposal">propose additional valuable features</a> you might still be missing - for me to review.','foodle-for-democracy-poll') ?></p>

  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Foodle Shortcode','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo $shortcode_parameters ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>[democracy id="12"]</td><td><?php echo __('Democracy Poll shortcode example','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>[foodle-democracy-poll-list-log id="12" show_date="false" show_category="true" categories="true" ml_single_sum="10" answerlist="false" categorysort="false" blocksort="false" solo="false" maxcount="0.1"]</td><td><?php echo __('Foodle shortcode use example','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle-democracy-poll-list-log</td><td><?php echo $the_shortcode_slug ?></td></tr>
      <tr><td>id="12"</td><td><?php echo __('Links to the Democracy Poll id','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>show_date="true/false"</td><td><?php echo $switch_on_off.': '.__('Display the column with the vote date (will be visible for selected viewers anyway)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>show_category="true/false"</td><td><?php echo $switch_on_off.': '.__('Display the column with the selected category','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>categories="true/false"</td><td><?php echo $switch_on_off.': '.__('Display selected categories (sorted) underneath the answer columns','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>ml_single_sum="10"</td><td><?php echo __('The number of rows (default=10) until which only one line containing the column\'s sum of votes shall be displayed','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>answerlist="true/false"</td><td><?php echo $switch_on_off.': '.__('Instead of displaying the answers as columns with green marks for each vote: list the answers of each voter as one column - this may be especially interesting, when individual answers of voters are enabled and displayed','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>categorysort="true/false"</td><td><?php echo $switch_on_off.': '.__('Decides whether to start the table list with sorting by category (yes) or the date of vote (false)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>blocksort="true/false"</td><td><?php echo $switch_on_off.': '.__('Blocks (true) or allows (false) to display the button for changing the initial sorting','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>solo="true/false"</td><td><?php echo $switch_on_off.': '.__('Display everything as selected but without the special viewers areas (reminders area & hidden date column), without the title line and without the \'foodle_table_top_spacing\' - \'foodle_table_bottom_spacing\' stays active','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>maxcount="x.y"</td><td><?php echo __('Highlight in green color the first x voters that voted for column y (if only one parameter is given, x.1 is assumed, i.e. x at first column, special roles/users will not be counted & highlighted if in the "marked" category) - one potential use could be to determine certain applications on a first-come/first-serve basis','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Foodle Poll Link Shortcode','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo $shortcode_parameters ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>[foodle-link-democracy-poll id="12" not_same="false" verbose="true" horizontal="left" status='logged-in, not-logged-in']</td><td><?php echo __('Foodle Poll Link shortcode use example.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>[foodle-link-democracy-poll id="self" not_same="true" verbose="false" horizontal="center" status='logged-in']</td><td><?php echo __('Foodle Poll Link shortcode use example inside the Democracy Poll\'s own text field.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle-link-democracy-poll</td><td><?php echo $the_shortcode_slug ?></td></tr>
      <tr><td>id="12"</td><td><?php echo __('Democracy Poll id.<br>The shortcode will generate an HTML-link to the first page/post referred to in the poll\'s database if there\'s one. For usage inside Democracy Poll\'s own text field, you may provide "self" as a valid value.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>not_same="true/false"</td><td><?php echo __('"true": No link dispay if the shortcode is on the main poll page, "false": Display the link in any case - default = "true".','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>verbose="true/false"</td><td><?php echo __('"true": Verbose mode: errors will be displayed, "false": Silent mode: errors will not be displayed - default = "true".','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>horizontal="left/center/right"</td><td><?php echo __('Defines the horizontal button alignment - default = "left".','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>status="logged-in, not-logged-in"</td><td><?php echo __('Defines the user status requirement list for the link button display - default = "logged-in, not-logged in", meaning: display in both cases.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td colspan="2"><?php echo __('This shortcode can be used anywhere. However, it could make sense near a remote representation of the poll shortcode (e.g. inside Democracy Poll\'s own text field - see above) or the Foodle shortcode.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td><?php echo __('One word on the link to the (main) poll page/post (see next shortcode)','foodle-for-democracy-poll') ?></td><td><?php echo __('Democracy Poll stores the pages/posts where it considers presence of its shortcode. This plugin will consider the first entry per poll to be the master (if it exists anyway).','foodle-for-democracy-poll') ?></td></tr>
      <tr><td><?php echo __('Known Limitations','foodle-for-democracy-poll') ?></td><td><?php echo __('Under conditions not fully understood, Democracy Poll will record the polls/posts with its shortcode not properly. This can be corrected in the database (PREFIXdemocracy_q, field \'in_posts\').','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Foodle Poll Statistics and Database Check Shortcode','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo $shortcode_parameters ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>[foodle-democracy-poll-database-check id="12" use="all"]</td><td><?php echo __('Foodle Poll Statistics and Database Check shortcode use example','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle-democracy-poll-database-check</td><td><?php echo $the_shortcode_slug ?></td></tr>
      <tr><td>id="12"</td><td><?php echo __('Democracy Poll id','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>use="statistics/check/all"</td><td><?php echo __('"statistics": show the poll statistics, "check": show the logical database check results, "all": show both','foodle-for-democracy-poll') ?></td></tr>
      <tr><td colspan="2"><?php echo __('The shortcode\'s complete function set is used as a dashboard widget. However you may use it elswhere and configured as you like, e.g. for other registered users.','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Foodle \'Display On For Roles\' Shortcode','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo $shortcode_parameters ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>[foodle-display-on-for-roles roles="aaa,bbb,..." marking="true"] ... content ... [/foodle-display-on-for-roles]</td><td><?php echo __('Foodle \'Display On For Roles\' shortcode use example: the shortcode requires a closing tag to contain the content to be controlled','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle-display-on-for-roles</td><td><?php echo $the_shortcode_slug ?></td></tr>
      <tr><td>roles="aaa,bbb,..."</td><td><?php echo __('A comma-separated list of roles for which the content is to be shown','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>marking="true/false"</td><td><?php echo $switch_on_off.': '.__('Whether to mark the content, indicating that it is hidden for others: in original-state, a dashed border is shown if \'true\' - you can influence the marking by your own CSS','foodle-for-democracy-poll') ?></td></tr>
      <tr><td colspan="2"><?php echo __('Use cases: e.g. the Democracy Poll archive, polls for certain user groups, or other uses. You can use this shortcode anywhere in your environment, not just for Foodle.','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Foodle \'Archive Do Not Show\' Shortcode','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo $shortcode_parameters ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>[foodle-archive-do-not-show do_not_show="aaa,bbb,..."] ... content ... [/foodle-archive-do-not-show]</td><td><?php echo __('Foodle \'Archive Do Not Show\' shortcode use example: the shortcode requires a closing tag to contain the content to be controlled','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle-archive-do-not-show</td><td><?php echo $the_shortcode_slug ?></td></tr>
      <tr><td>do_not_show="aaa,bbb,..."</td><td><?php echo __('A comma-separated list of Foodle IDs for which the display in the content is to be inhibited','foodle-for-democracy-poll') ?></td></tr>
      <tr><td colspan="2"><?php echo __('Use case: the content needs to be the Democracy Poll archive shortcode, e.g. [democracy_archives active="1"], for which the display of certain Foodle IDs will be inhibited. This can be useful to display certain archive excerpts for specific user groups or avoid the display of private polls. There is no other option for this shortcode\'s content!','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Foodle \'(Help- and Warning-)Tooltips\'','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo __('Usage','foodle-for-democracy-poll') ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>foodle_tooltip="..."</td><td><?php echo __('Usage inside an HTML-tag as required (... = your text with optional HTML tags)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_tooltip_delay="a"</td><td><?php echo __('The delay to show the tooltip in a ms (default if missing = 400 ms)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_tooltip_dy="b"</td><td><?php echo __('The vertical distance to the mouse-pointer in b px (default if missing = 14 px)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_tooltip_dx="c"</td><td><?php echo __('The horizontal distance to the mouse-pointer in c px (default if missing = 14 px)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>#foodle_jqueryTooltip</td><td><?php echo __('The related CSS id to control its styles','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_ttwarning="..."</td><td><?php echo __('Usage inside an HTML-tag as required (... = your text with optional HTML tags)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_ttwarning_delay="x"</td><td><?php echo __('The delay to show the tooltip in x ms (default if missing = 0 ms)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_ttwarning_dy="y"</td><td><?php echo __('The vertical distance to the mouse-pointer in y px (default if missing = -80 px)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>foodle_ttwarning_dx="z"</td><td><?php echo __('The horizontal distance to the mouse-pointer in z px (default if missing = -20 px)','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>#foodle_jqueryTtwarning</td><td><?php echo __('The related CSS id to control its styles','foodle-for-democracy-poll') ?></td></tr>
      <tr><td colspan="2"><?php echo __('Can be used anywhere in the front-end and the back-end.','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('How To Handle Your Own CSS Styles','foodle-for-democracy-poll') ?></p>
  <figure class="foodle-block-table">
    <table class="foodle-tips-table"><tbody>
      <tr class="foodle-header-row"><th style="width:33.34%; min-width:260px;"><?php echo __('Classes','foodle-for-democracy-poll') ?></th><th style="width:66.66%;"><?php echo $explanations ?></th></tr>
      <tr><td>.democracy</td><td><?php echo __('The Democracy Poll front-end in general. The main place to handle the design is the tab \'Theme Settings\' in Democracy Poll\'s administration pages. Foodle\'s (friendly) CSS allows you to also remove or adjust the design changes done by Foodle. The default Foodle CSS is based on the Democracy \'block.css\' selection in \'Theme Settings\').','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.democracy-voting</td><td><?php echo __('The Democracy Poll front-end when voting. For all other explanations: see class \'.democracy\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.democracy-results</td><td><?php echo __('The Democracy Poll front-end when displaying the voting results. For all other explanations: see class \'.democracy\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.democracy-voted</td><td><?php echo __('The Democracy Poll front-end when the viewing user did already vote for this poll. For all other explanations: see class \'.democracy\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.democracy-not-voted</td><td><?php echo __('The Democracy Poll front-end when the viewing user did not yet vote for this poll. For all other explanations: see class \'.democracy\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.democracy-your-vote</td><td><?php echo __('The text in the results display that indicates where the viewing user placed his votes.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.democracy-refresh-button</td><td><?php echo __('The button to start an AJAX refresh of the Democracy Poll results display.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-line</td><td><?php echo __('Foodle\'s horizontal line.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-block-table</td><td><?php echo __('Used in the &lt;figure&gt; tag that encloses the Foodle table. In original state, it allows the Foodle table to be scrolled horizontally on small screens.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.is-style-stripes</td><td><?php echo __('Used in the &lt;figure&gt; tag for the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.has-subtle-light-grey-background-color</td><td><?php echo __('Used in the &lt;figure&gt; tag for the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-table</td><td><?php echo __('The Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.category-inner-div</td><td><?php echo __('The inner category display underneath each answer column. In original state: just centered horizontally.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-button</td><td><?php echo __('The button in the email-reminder area.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-sort-is</td><td><?php echo __('The information above the Foodle Table\'s sort button, indicating the current sorting mode. In original state: "white-space:nowrap;" plus a few others.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-sort-button</td><td><?php echo __('The button to switch sorting modes in the Foodle table. In original state: "white-space:nowrap;" plus a few others.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-link-button</td><td><?php echo __('The button used by shortcode \'foodle-link-democracy-poll\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-democracy-top-spacing</td><td><?php echo __('The spacing above the Democracy Poll shortcode.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-table-hidden-top-spacing</td><td><?php echo __('The spacing above the hidden reminder area.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-table-hidden-spacing</td><td><?php echo __('The spacing inside the hidden reminder area towards the horizontal lines.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-table-top-spacing</td><td><?php echo __('The spacing above the Foodle table. Unused when solo="true".','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-table-headline</td><td><?php echo __('The headline above the Foodle table. You might want to change it or switch it off independently from the shortcode parameter \'solo="true"\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-table-bottom-spacing</td><td><?php echo __('The spacing below the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-voters-header</td><td><?php echo __('The header of the voters column in the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-datcol-header</td><td><?php echo __('The header of the date column in the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-category-header</td><td><?php echo __('The header of the category column in the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-answer-header</td><td><?php echo __('The header of the answers column(s) in the Foodle table.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-info-rows</td><td><?php echo __('The Foodle table cells providing information except the very header row and except the category cells underneath the answers.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-category-cells</td><td><?php echo __('The Foodle table category cells underneath the answers.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-category-cells-maxcount-answer</td><td><?php echo __('The Foodle table category cell underneath an answer when \'maxcount\' is counted for this answer.<br>Reason: The category sum will only consider the answers up to the \'maxcount\'.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-cell-is-marked</td><td><?php echo __('The Foodle table cells with marking.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-cell-is-in</td><td><?php echo __('The Foodle table cells marked as being inside the maximum count given for \'votes\' in one of the answers.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-cell-is-marked-in</td><td><?php echo __('The Foodle table name cells marked when a marked user (see above list of functional additions to Democracy Poll and <a href="/wp-admin/options-general.php?page=foodle-admin-page&tab=special-roles-users">\'Special Roles & Users\'</a>) is being counted. In original state: a gradient backgound color.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-vertical</td><td><?php echo __('For the date column. In original state: Just for font-size, angle not used (0deg).','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-display-on-with-marking</td><td><?php echo __('The hidden content marking if switched on (see \'Foodle Display On For Roles\' shortcode). In original state: a dashed border.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-display-on-no-marking</td><td><?php echo __('The same hidden content not marked. In original state: no CSS defined.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td>.foodle-statistics-check-table</td><td><?php echo __('The related dashboard widget / shortcode table. In original state: not used.','foodle-for-democracy-poll') ?></td></tr>
      <tr><td><?php echo __('Where to put my CSS','foodle-for-democracy-poll') ?>?</td><td><?php echo __('In the Democracy Poll administration pages, there is a tab called \'Theme Settings\' with a chapter \'Custom/Addition CSS styles\'.<br>This is the area to insert your own CSS, which can cover both, Democracy Poll and Foodle.','foodle-for-democracy-poll') ?></td></tr>
    </tbody></table>
  </figure>
  <p>&nbsp;</p>
  <p class="foodle-headline"><?php echo __('The Democracy Poll Edit Page Updates','foodle-for-democracy-poll') ?></p>
  <p class="foodle-text"><img style="border: 1px solid SteelBlue;" src="<?php echo plugin_dir_url(__FILE__) ?>img/democracy-edit-new.png"<? echo $help_tips_edit_pic ?>width="900"></p>
  <p>&nbsp;</p>
  <p>&nbsp;</p>
  <p style="font-size:1.2em;"><?php echo $foodle_like_me_1 ?><img src="<?php echo plugin_dir_url( __FILE__ ) ?>img/PayPal-Logo-2019-kl.jpg" height="16" alt="PayPal"><?php echo $foodle_like_me_2 ?></p>
  <form action="https://www.paypal.com/donate" method="post" target="_top">
  <input type="hidden" name="hosted_button_id" value="W3V5CKXFJS948" />
  <input type="image" src="https://www.paypalobjects.com/en_US/DK/i/btn/btn_donateCC_LG.gif" border="0" name="submit" title="PayPal - The safer, easier way to pay online!" alt="Donate with PayPal button" />
  <img alt="" border="0" src="https://www.paypal.com/en_DE/i/scr/pixel.gif" width="1" height="1" />
  </form>
  <?php
}



// The success notice as admin_notice...
function foodle_pwx_admin_notice__success($foodle_success_message) {
  ?>
  <div class="notice notice-success is-dismissible">
    <?php
      echo '<p>&nbsp;</p>';
      echo $foodle_success_message;
      echo '<p>&nbsp;</p>';
    ?>
  </div>
  <?php
}



// The defined error notice(s) as admin_notice...
function foodle_pwx_admin_notice__defined_error($foodle_error_message) {
  ?>
  <div class="notice notice-error is-dismissible">
    <?php
      echo '<p>&nbsp;</p>';
      echo _e('ERROR','foodle-for-democracy-poll').': '.$foodle_error_message;
      echo '<p>&nbsp;</p>';
    ?>
  </div>
  <?php
}



// The undefined error notice as admin_notice...
function foodle_pwx_admin_notice__error() {
  ?>
  <div class="notice notice-error is-dismissible">
    <?php
      echo '<h2 class="mfem-h2">'.__( 'ERROR: The action was most probably not properly performed!', 'foodle-for-democracy-poll' ).'</h2>';
    ?>
  </div>
  <?php
}



// The last-chance-cancel warning(s) as admin_notice...
function foodle_pwx_admin_notice__warning($foodle_warning_message) {
  ?>
  <div class="notice notice-warning is-dismissible">
    <?php
      echo '<p>&nbsp;</p>';
      echo $foodle_warning_message;
      echo '<p>&nbsp;</p>';
    ?>
  </div>
  <?php
}



// Check whether Democracy will keep logs. Otherwise, issue a warning!
function foodle_check_keeping_logs() {
  if ( isset(get_option("democracy_options")["keep_logs"]) ) {
    if (! get_option("democracy_options")["keep_logs"] )
      return "<div class='notice error notice-error is-dismissible missing-dem-logs' style='padding-top:10px;padding-bottom:10px;'>".__('<strong>WARNING!</strong><br>Foodle cannot work, as Democracy Poll will not keep voting logs in the database.<br>Please change the related Democracy Poll setting if you want to use Foodle!<br>Storage of IP addresses can be inhibited separately per poll.','foodle-for-democracy-poll')."</div>";
  }
  return "";
}


